@@ -12,9 +12,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from __future__ import absolute_import
-from __future__ import print_function
-
+import argparse
+import datetime
 import os
 import sys
 import uuid
@@ -30,6 +29,7 @@ from keystone.cmd import bootstrap
 from keystone.cmd import doctor
 from keystone.common import driver_hints
 from keystone.common import fernet_utils
+from keystone.common import jwt_utils
 from keystone.common import sql
 from keystone.common.sql import upgrades
 from keystone.common import utils
@@ -41,7 +41,6 @@ from keystone.federation import utils as mapping_engine
 from keystone.i18n import _
 from keystone.server import backends
 
-
 CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
@@ -94,7 +93,7 @@ class BootStrap(BaseApp):
                             metavar='OS_BOOTSTRAP_ADMIN_URL',
                             help=('The initial identity admin url created '
                                   'during the keystone bootstrap process. '
-                                  'e.g. http://127.0.0.1:35357/v3'))
+                                  'e.g. http://127.0.0.1:5000/v3'))
         parser.add_argument('--bootstrap-public-url',
                             metavar='OS_BOOTSTRAP_PUBLIC_URL',
                             help=('The initial identity public url created '
@@ -110,6 +109,20 @@ class BootStrap(BaseApp):
                             help=('The initial region_id endpoints will be '
                                   'placed in during the keystone bootstrap '
                                   'process.'))
+        parser.add_argument('--immutable-roles',
+                            default=True,
+                            action='store_true',
+                            help=('Whether default roles (admin, member, and '
+                                  'reader) should be immutable. This is the '
+                                  'default.'))
+        parser.add_argument('--no-immutable-roles',
+                            default=False,
+                            action='store_true',
+                            help=('Whether default roles (admin, member, and '
+                                  'reader) should be immutable. Immutable '
+                                  'default roles is the default, use this '
+                                  'flag to opt out of immutable default '
+                                  'roles.'))
         return parser
 
     def do_bootstrap(self):
@@ -151,9 +164,9 @@ class BootStrap(BaseApp):
         self.endpoints = None
 
         if self.password is None:
-            print(_('Either --bootstrap-password argument or '
+            print(_('ERROR: Either --bootstrap-password argument or '
                     'OS_BOOTSTRAP_PASSWORD must be set.'))
-            raise ValueError
+            sys.exit(1)
 
         self.bootstrapper.admin_password = self.password
         self.bootstrapper.admin_username = self.username
@@ -165,6 +178,10 @@ class BootStrap(BaseApp):
         self.bootstrapper.public_url = self.public_url
         self.bootstrapper.internal_url = self.internal_url
         self.bootstrapper.region_id = self.region_id
+        if CONF.command.no_immutable_roles:
+            self.bootstrapper.immutable_roles = False
+        else:
+            self.bootstrapper.immutable_roles = True
 
         self.bootstrapper.bootstrap()
         self.reader_role_id = self.bootstrapper.reader_role_id
@@ -252,7 +269,7 @@ class DbSync(BaseApp):
         return parser
 
     @classmethod
-    def check_db_sync_status(self):
+    def check_db_sync_status(cls):
         status = 0
         try:
             expand_version = upgrades.get_db_version(repo='expand_repo')
@@ -383,13 +400,40 @@ class BasePermissionsSetup(BaseApp):
 
         return keystone_user_id, keystone_group_id
 
+    @classmethod
+    def initialize_fernet_repository(
+            cls, keystone_user_id, keystone_group_id, config_group=None):
+        conf_group = getattr(CONF, config_group)
+        futils = fernet_utils.FernetUtils(
+            conf_group.key_repository,
+            conf_group.max_active_keys,
+            config_group
+        )
+
+        futils.create_key_directory(keystone_user_id, keystone_group_id)
+        if futils.validate_key_repository(requires_write=True):
+            futils.initialize_key_repository(
+                keystone_user_id, keystone_group_id)
+
+    @classmethod
+    def rotate_fernet_repository(
+            cls, keystone_user_id, keystone_group_id, config_group=None):
+        conf_group = getattr(CONF, config_group)
+        futils = fernet_utils.FernetUtils(
+            conf_group.key_repository,
+            conf_group.max_active_keys,
+            config_group
+        )
+        if futils.validate_key_repository(requires_write=True):
+            futils.rotate_keys(keystone_user_id, keystone_group_id)
+
 
 class FernetSetup(BasePermissionsSetup):
-    """Setup a key repository for Fernet tokens.
+    """Setup key repositories for Fernet tokens and auth receipts.
 
     This also creates a primary key used for both creating and validating
-    Fernet tokens. To improve security, you should rotate your keys (using
-    keystone-manage fernet_rotate, for example).
+    Fernet tokens and auth receipts. To improve security, you should rotate
+    your keys (using keystone-manage fernet_rotate, for example).
 
     """
 
@@ -397,17 +441,27 @@ class FernetSetup(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        futils = fernet_utils.FernetUtils(
-            CONF.fernet_tokens.key_repository,
-            CONF.fernet_tokens.max_active_keys,
-            'fernet_tokens'
-        )
-
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        futils.create_key_directory(keystone_user_id, keystone_group_id)
-        if futils.validate_key_repository(requires_write=True):
-            futils.initialize_key_repository(
-                keystone_user_id, keystone_group_id)
+        cls.initialize_fernet_repository(
+            keystone_user_id, keystone_group_id, 'fernet_tokens')
+
+        if (os.path.abspath(CONF.fernet_tokens.key_repository) !=
+                os.path.abspath(CONF.fernet_receipts.key_repository)):
+            cls.initialize_fernet_repository(
+                keystone_user_id, keystone_group_id, 'fernet_receipts')
+        elif(CONF.fernet_tokens.max_active_keys !=
+                CONF.fernet_receipts.max_active_keys):
+            # WARNING(adriant): If the directories are the same,
+            # 'max_active_keys' is ignored from fernet_receipts in favor of
+            # fernet_tokens to avoid a potential mismatch. Only if the
+            # directories are different do we create a different one for
+            # receipts, and then respect 'max_active_keys' for receipts.
+            LOG.warning(
+                "Receipt and Token fernet key directories are the same "
+                "but `max_active_keys` is different. Receipt "
+                "`max_active_keys` will be ignored in favor of Token "
+                "`max_active_keys`."
+            )
 
 
 class FernetRotate(BasePermissionsSetup):
@@ -432,15 +486,50 @@ class FernetRotate(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        futils = fernet_utils.FernetUtils(
-            CONF.fernet_tokens.key_repository,
-            CONF.fernet_tokens.max_active_keys,
-            'fernet_tokens'
-        )
-
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        if futils.validate_key_repository(requires_write=True):
-            futils.rotate_keys(keystone_user_id, keystone_group_id)
+        cls.rotate_fernet_repository(
+            keystone_user_id, keystone_group_id, 'fernet_tokens')
+        if (os.path.abspath(CONF.fernet_tokens.key_repository) !=
+                os.path.abspath(CONF.fernet_receipts.key_repository)):
+            cls.rotate_fernet_repository(
+                keystone_user_id, keystone_group_id, 'fernet_receipts')
+
+
+class CreateJWSKeyPair(BasePermissionsSetup):
+    """Create a key pair for signing and validating JWS tokens.
+
+    This command creates a public and private key pair to use for signing and
+    validating JWS token signatures. The key pair is written to the directory
+    where the command is invoked.
+
+    """
+
+    name = 'create_jws_keypair'
+
+    @classmethod
+    def add_argument_parser(cls, subparsers):
+        parser = super(CreateJWSKeyPair, cls).add_argument_parser(subparsers)
+
+        parser.add_argument(
+            '--force', action='store_true',
+            help=('Forcibly overwrite keys if they already exist')
+        )
+        return parser
+
+    @classmethod
+    def main(cls):
+        current_directory = os.getcwd()
+        private_key_path = os.path.join(current_directory, 'private.pem')
+        public_key_path = os.path.join(current_directory, 'public.pem')
+
+        if os.path.isfile(private_key_path) and not CONF.command.force:
+            raise SystemExit(_('Private key %(path)s already exists')
+                             % {'path': private_key_path})
+        if os.path.isfile(public_key_path) and not CONF.command.force:
+            raise SystemExit(_('Public key %(path)s already exists')
+                             % {'path': public_key_path})
+
+        jwt_utils.create_jws_keypair(private_key_path, public_key_path)
 
 
 class TokenSetup(BasePermissionsSetup):
@@ -456,18 +545,9 @@ class TokenSetup(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        futils = fernet_utils.FernetUtils(
-            # TODO(gagehugo) Change this to CONF.token
-            CONF.fernet_tokens.key_repository,
-            CONF.fernet_tokens.max_active_keys,
-            'fernet_tokens'
-        )
-
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        futils.create_key_directory(keystone_user_id, keystone_group_id)
-        if futils.validate_key_repository(requires_write=True):
-            futils.initialize_key_repository(
-                keystone_user_id, keystone_group_id)
+        cls.initialize_fernet_repository(
+            keystone_user_id, keystone_group_id, 'fernet_tokens')
 
 
 class TokenRotate(BasePermissionsSetup):
@@ -492,16 +572,54 @@ class TokenRotate(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        futils = fernet_utils.FernetUtils(
-            # TODO(gagehugo) Change this to CONF.token
-            CONF.fernet_tokens.key_repository,
-            CONF.fernet_tokens.max_active_keys,
-            'fernet_tokens'
-        )
-
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        if futils.validate_key_repository(requires_write=True):
-            futils.rotate_keys(keystone_user_id, keystone_group_id)
+        cls.rotate_fernet_repository(
+            keystone_user_id, keystone_group_id, 'fernet_tokens')
+
+
+class ReceiptSetup(BasePermissionsSetup):
+    """Setup a key repository for auth receipts.
+
+    This also creates a primary key used for both creating and validating
+    receipts. To improve security, you should rotate your keys (using
+    keystone-manage receipt_rotate, for example).
+
+    """
+
+    name = 'receipt_setup'
+
+    @classmethod
+    def main(cls):
+        keystone_user_id, keystone_group_id = cls.get_user_group()
+        cls.initialize_fernet_repository(
+            keystone_user_id, keystone_group_id, 'fernet_receipts')
+
+
+class ReceiptRotate(BasePermissionsSetup):
+    """Rotate auth receipts encryption keys.
+
+    This assumes you have already run keystone-manage receipt_setup.
+
+    A new primary key is placed into rotation, which is used for new receipts.
+    The old primary key is demoted to secondary, which can then still be used
+    for validating receipts. Excess secondary keys (beyond [receipt]
+    max_active_keys) are revoked. Revoked keys are permanently deleted. A new
+    staged key will be created and used to validate receipts. The next time key
+    rotation takes place, the staged key will be put into rotation as the
+    primary key.
+
+    Rotating keys too frequently, or with [receipt] max_active_keys set
+    too low, will cause receipts to become invalid prior to their expiration.
+
+    """
+
+    name = 'receipt_rotate'
+
+    @classmethod
+    def main(cls):
+        keystone_user_id, keystone_group_id = cls.get_user_group()
+        cls.rotate_fernet_repository(
+            keystone_user_id, keystone_group_id, 'fernet_receipts')
 
 
 class CredentialSetup(BasePermissionsSetup):
@@ -659,20 +777,57 @@ class CredentialMigrate(BasePermissionsSetup):
         klass.migrate_credentials()
 
 
-class TokenFlush(BaseApp):
-    """Flush expired tokens from the backend."""
+class TrustFlush(BaseApp):
+    """Flush expired and non-expired soft deleted trusts from the backend."""
 
-    name = 'token_flush'
+    name = 'trust_flush'
+
+    @classmethod
+    def add_argument_parser(cls, subparsers):
+        parser = super(TrustFlush, cls).add_argument_parser(subparsers)
+
+        parser.add_argument('--project-id', default=None,
+                            help=('The id of the project of which the '
+                                  'expired or non-expired soft-deleted '
+                                  'trusts is to be purged'))
+        parser.add_argument('--trustor-user-id', default=None,
+                            help=('The id of the trustor of which the '
+                                  'expired or non-expired soft-deleted '
+                                  'trusts is to be purged'))
+        parser.add_argument('--trustee-user-id', default=None,
+                            help=('The id of the trustee of which the '
+                                  'expired or non-expired soft-deleted '
+                                  'trusts is to be purged'))
+        parser.add_argument('--date', default=datetime.datetime.utcnow(),
+                            help=('The date of which the expired or '
+                                  'non-expired soft-deleted trusts older '
+                                  'than that will be purged. The format of '
+                                  'the date to be "DD-MM-YYYY". If no date '
+                                  'is supplied keystone-manage will use the '
+                                  'system clock time at runtime'))
+        return parser
 
     @classmethod
     def main(cls):
-        LOG.warning(
-            'The token_flush command is deprecated and no longer needed with '
-            'the development of non-persistent token formats. It will be '
-            'removed in Stein. It is recommended that you remove usage of '
-            'this command or integrate it\'s functionality into a separate '
-            'tool if you are using an out-of-tree provider that relies on '
-            'persistent token storage.'
+        drivers = backends.load_backends()
+        trust_manager = drivers['trust_api']
+        if CONF.command.date:
+            if not isinstance(CONF.command.date, datetime.datetime):
+                try:
+                    CONF.command.date = datetime.datetime.strptime(
+                        CONF.command.date, '%d-%m-%Y')
+                except KeyError:
+                    raise ValueError("'%s'Invalid input for date, should be "
+                                     "DD-MM-YYYY", CONF.command.date)
+            else:
+                LOG.info("No date is supplied, keystone-manage will use the "
+                         "system clock time at runtime ")
+
+        trust_manager.flush_expired_and_soft_deleted_trusts(
+            project_id=CONF.command.project_id,
+            trustor_user_id=CONF.command.trustor_user_id,
+            trustee_user_id=CONF.command.trustee_user_id,
+            date=CONF.command.date
         )
 
 
@@ -863,7 +1018,7 @@ class DomainConfigUploadFiles(object):
         try:
             for group in sections:
                 for option in sections[group]:
-                        sections[group][option] = sections[group][option][0]
+                    sections[group][option] = sections[group][option][0]
             self.domain_config_manager.create_config(domain_ref['id'],
                                                      sections)
             return True
@@ -990,7 +1145,7 @@ class MappingEngineTester(BaseApp):
     def read_rules(self, path):
         self.rules_pathname = path
         try:
-            with open(path) as file:
+            with open(path, "rb") as file:
                 self.rules = jsonutils.load(file)
         except ValueError as e:
             raise SystemExit(_('Error while parsing rules '
@@ -1017,7 +1172,7 @@ class MappingEngineTester(BaseApp):
                 raise SystemExit(msg % {'pathname': self.assertion_pathname,
                                         'line_num': line_num,
                                         'line': line})
-        assertion = self.assertion.split('\n')
+        assertion = self.assertion.splitlines()
         assertion_dict = {}
         prefix = CONF.command.prefix
         for line_num, line in enumerate(assertion, 1):
@@ -1068,26 +1223,28 @@ class MappingEngineTester(BaseApp):
         parser = super(MappingEngineTester,
                        cls).add_argument_parser(subparsers)
 
+        parser.formatter_class = argparse.RawTextHelpFormatter
         parser.add_argument('--rules', default=None, required=True,
                             help=("Path to the file with "
                                   "rules to be executed. "
-                                  "Content must be a proper JSON structure, "
-                                  "with a top-level key 'rules' and "
+                                  "Content must be\na proper JSON structure, "
+                                  "with a top-level key 'rules' and\n"
                                   "corresponding value being a list."))
         parser.add_argument('--input', default=None, required=True,
                             help=("Path to the file with input attributes. "
-                                  "The content consists of ':' separated "
-                                  "parameter names and their values. "
-                                  "There is only one key-value pair per line. "
-                                  "A ';' in the value is a separator and then "
-                                  "a value is treated as a list. Example:\n "
-                                  "EMAIL: me@example.com\n"
-                                  "LOGIN: me\n"
-                                  "GROUPS: group1;group2;group3"))
+                                  "The content\nconsists of ':' separated "
+                                  "parameter names and their values.\nThere "
+                                  "is only one key-value pair per line. "
+                                  "A ';' in the\nvalue is a separator and "
+                                  "then a value is treated as a list.\n"
+                                  "Example:\n"
+                                  "\tEMAIL: me@example.com\n"
+                                  "\tLOGIN: me\n"
+                                  "\tGROUPS: group1;group2;group3"))
         parser.add_argument('--prefix', default=None,
                             help=("A prefix used for each environment "
-                                  "variable in the assertion. For example, "
-                                  "all environment variables may have the "
+                                  "variable in the\nassertion. For example, "
+                                  "all environment variables may have\nthe "
                                   "prefix ASDF_."))
         parser.add_argument('--engine-debug',
                             default=False, action="store_true",
@@ -1152,13 +1309,16 @@ CMDS = [
     DomainConfigUpload,
     FernetRotate,
     FernetSetup,
+    CreateJWSKeyPair,
     MappingPopulate,
     MappingPurge,
     MappingEngineTester,
+    ReceiptRotate,
+    ReceiptSetup,
     SamlIdentityProviderMetadata,
-    TokenFlush,
     TokenRotate,
-    TokenSetup
+    TokenSetup,
+    TrustFlush
 ]
 
 
