@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import copy
 import functools
 
 import flask
@@ -20,7 +19,6 @@ from oslo_utils import strutils
 
 from keystone.common import authorization
 from keystone.common import context
-from keystone.common import controller
 from keystone.common import policies
 from keystone.common import provider_api
 from keystone.common import utils
@@ -47,12 +45,48 @@ class RBACEnforcer(object):
     __shared_state__ = {}
     __ENFORCER = None
     ACTION_STORE_ATTR = 'keystone:RBAC:action_name'
+    # FOR TESTS ONLY
+    suppress_deprecation_warnings = False
 
     def __init__(self):
         # NOTE(morgan): All Enforcer Instances use the same shared state;
         # BORG pattern.
         self.__dict__ = self.__shared_state__
 
+    def _check_deprecated_rule(self, action):
+        def _name_is_changing(rule):
+            deprecated_rule = rule.deprecated_rule
+            return (deprecated_rule and
+                    deprecated_rule.name != rule.name and
+                    deprecated_rule.name in self._enforcer.file_rules)
+
+        def _check_str_is_changing(rule):
+            deprecated_rule = rule.deprecated_rule
+            return (deprecated_rule and
+                    deprecated_rule.check_str != rule.check_str and
+                    rule.name not in self._enforcer.file_rules)
+
+        def _is_deprecated_for_removal(rule):
+            return (rule.deprecated_for_removal and
+                    rule.name in self._enforcer.file_rules)
+
+        def _emit_warning():
+            if not self._enforcer._warning_emitted:
+                LOG.warning("Deprecated policy rules found. Use "
+                            "oslopolicy-policy-generator and "
+                            "oslopolicy-policy-upgrade to detect and resolve "
+                            "deprecated policies in your configuration.")
+                self._enforcer._warning_emitted = True
+
+        registered_rule = self._enforcer.registered_rules.get(action)
+
+        if not registered_rule:
+            return
+        if (_name_is_changing(registered_rule) or
+                _check_str_is_changing(registered_rule) or
+                _is_deprecated_for_removal(registered_rule)):
+            _emit_warning()
+
     def _enforce(self, credentials, action, target, do_raise=True):
         """Verify that the action is valid on the target in this context.
 
@@ -79,22 +113,11 @@ class RBACEnforcer(object):
             extra.update(exc=exception.ForbiddenAction, action=action,
                          do_raise=do_raise)
 
-        # NOTE(lbragstad): If there is a token in the credentials dictionary,
-        # it's going to be an instance of a TokenModel. We'll need to convert
-        # it to the a token response or dictionary before passing it to
-        # oslo.policy for enforcement. This is because oslo.policy shouldn't
-        # know how to deal with an internal object only used within keystone.
-        if 'token' in credentials:
-            token_ref = controller.render_token_response_from_model(
-                credentials['token']
-            )
-            credentials_copy = copy.deepcopy(credentials)
-            credentials_copy['token'] = token_ref
-            credentials = credentials_copy
-
         try:
-            return self._enforcer.enforce(
+            result = self._enforcer.enforce(
                 rule=action, target=target, creds=credentials, **extra)
+            self._check_deprecated_rule(action)
+            return result
         except common_policy.InvalidScope:
             raise exception.ForbiddenAction(action=action)
 
@@ -107,7 +130,22 @@ class RBACEnforcer(object):
         # The raw oslo-policy enforcer object
         if self.__ENFORCER is None:
             self.__ENFORCER = common_policy.Enforcer(CONF)
+            # NOTE(cmurphy) when running in the keystone server, suppress
+            # deprecation warnings for individual policy rules. Instead, we log
+            # a single notification at enforcement time indicating the
+            # oslo.policy tools the operator can use to detect and resolve
+            # deprecated policies. If there is no request context here, that
+            # means external tooling such as the oslo.policy tools are running
+            # this code, in which case we do want the full deprecation warnings
+            # emitted for individual polcy rules.
+            if flask.has_request_context():
+                self.__ENFORCER.suppress_deprecation_warnings = True
+            # NOTE(cmurphy) Tests may explicitly disable these warnings to
+            # prevent an explosion of test logs
+            if self.suppress_deprecation_warnings:
+                self.__ENFORCER.suppress_deprecation_warnings = True
             self.register_rules(self.__ENFORCER)
+            self.__ENFORCER._warning_emitted = False
         return self.__ENFORCER
 
     @staticmethod
@@ -161,7 +199,15 @@ class RBACEnforcer(object):
                 # here.
                 resource = flask.current_app.view_functions[
                     flask.request.endpoint].view_class
-                member_name = getattr(resource, 'member_name', None)
+                try:
+                    member_name = getattr(resource, 'member_key', None)
+                except ValueError:
+                    # NOTE(morgan): In the case that the ResourceBase keystone
+                    # class is used, we raise a value error when member_key
+                    # has not been set on the class. This is perfectly
+                    # normal and acceptable. Set member_name to None as though
+                    # it wasn't set.
+                    member_name = None
                 func = getattr(
                     resource, 'get_member_from_driver', None)
                 if member_name is not None and callable(func):
@@ -177,9 +223,8 @@ class RBACEnforcer(object):
                         # TODO(morgan): add (future) support for passing class
                         # instantiation args.
                         ret_dict['target'] = {
-                            member_name: func(
-                                resource(),
-                                flask.request.view_args[key])[member_name]}
+                            member_name: func(flask.request.view_args[key])
+                        }
         return ret_dict
 
     @staticmethod
@@ -197,6 +242,8 @@ class RBACEnforcer(object):
         # of the auth paths.
         target = 'token'
         subject_token = flask.request.headers.get('X-Subject-Token')
+        access_rules_support = flask.request.headers.get(
+            authorization.ACCESS_RULES_HEADER)
         if subject_token is not None:
             allow_expired = (strutils.bool_from_string(
                 flask.request.args.get('allow_expired', False),
@@ -205,13 +252,14 @@ class RBACEnforcer(object):
                 window_seconds = CONF.token.allow_expired_window
             token = PROVIDER_APIS.token_provider_api.validate_token(
                 subject_token,
-                window_seconds=window_seconds
+                window_seconds=window_seconds,
+                access_rules_support=access_rules_support
             )
             # TODO(morgan): Expand extracted data from the subject token.
             ret_dict[target] = {}
             ret_dict[target]['user_id'] = token.user_id
             try:
-                user_domain_id = token.user_domain_id
+                user_domain_id = token.user['domain_id']
             except exception.UnexpectedError:
                 user_domain_id = None
             if user_domain_id:
@@ -246,7 +294,7 @@ class RBACEnforcer(object):
     @classmethod
     def enforce_call(cls, enforcer=None, action=None, target_attr=None,
                      member_target_type=None, member_target=None,
-                     filters=None):
+                     filters=None, build_target=None):
         """Enforce RBAC on the current request.
 
         This will do some legwork and then instantiate the Enforcer if an
@@ -277,12 +325,17 @@ class RBACEnforcer(object):
         :type member_target: dict
         :param filters: A variable number of optional string filters, these are
                         used to extract values from the query params. The
-                        filters are added to the reques data that is passed to
+                        filters are added to the request data that is passed to
                         the enforcer and may be used to determine policy
                         action. In practice these are mainly supplied in the
                         various "list" APIs and are un-used in the default
                         supplied policies.
         :type filters: iterable
+        :param build_target: A function to build the target for enforcement.
+                             This is explicitly done after authentication
+                             in order to not leak existance data before
+                             auth.
+        :type build_target: function
         """
         # NOTE(morgan) everything in the policy_dict may be used by the policy
         # DSL to action on RBAC and request information/response data.
@@ -336,9 +389,26 @@ class RBACEnforcer(object):
         policy_dict.update(flask.request.view_args)
 
         # Get the Target Data Set.
-        if target_attr is None:
-            policy_dict.update(cls._extract_member_target_data(
-                member_target_type, member_target))
+        if target_attr is None and build_target is None:
+            try:
+                policy_dict.update(cls._extract_member_target_data(
+                    member_target_type, member_target))
+            except exception.NotFound:
+                # DEBUG LOG and bubble up the 404 error. This is expected
+                # behavior. This likely should be specific in each API. This
+                # should be revisited in the future and each API should make
+                # the explicit "existence" checks before enforcement.
+                LOG.debug('Extracting inferred target data resulted in '
+                          '"NOT FOUND (404)".')
+                raise
+            except Exception as e:  # nosec
+                # NOTE(morgan): Errors should never bubble up at this point,
+                # if there is an error getting the target, log it and move
+                # on. Raise an explicit 403, we have failed policy checks.
+                LOG.warning('Unable to extract inferred target data during '
+                            'enforcement')
+                LOG.debug(e, exc_info=True)
+                raise exception.ForbiddenAction(action=action)
 
             # Special Case, extract and add subject_token data.
             subj_token_target_data = cls._extract_subject_token_target_data()
@@ -346,7 +416,11 @@ class RBACEnforcer(object):
                 policy_dict.setdefault('target', {}).update(
                     subj_token_target_data)
         else:
-            policy_dict['target'] = target_attr
+            if target_attr and build_target:
+                raise ValueError('Programming Error: A target_attr or '
+                                 'build_target must be provided, but not both')
+
+            policy_dict['target'] = target_attr or build_target()
 
         # Pull the data from the submitted json body to generate
         # appropriate input/target attributes, we take an explicit copy here
@@ -357,8 +431,6 @@ class RBACEnforcer(object):
         # Generate the filter_attr dataset.
         policy_dict.update(cls._extract_filter_values(filters))
 
-        # Extract the cred data
-        creds = cls._extract_policy_check_credentials()
         flattened = utils.flatten_dict(policy_dict)
         if LOG.logger.getEffectiveLevel() <= log.DEBUG:
             # LOG the Args
@@ -369,26 +441,11 @@ class RBACEnforcer(object):
             LOG.debug('RBAC: Authorizing `%(action)s(%(args)s)`',
                       {'action': action, 'args': args_str})
 
-            # LOG the Cred Data
-            cred_str = ', '.join(
-                ['%s=%s' % (k, v) for k, v in creds.items()])
-            cred_str = strutils.mask_password(cred_str)
-            LOG.debug('RBAC: Policy Enforcement Cred Data '
-                      '`%(action)s creds(%(cred_str)s)`',
-                      {'action': action, 'cred_str': cred_str})
-
-            # Log the Target Data
-            target_str = ', '.join(
-                ['%s=%s' % (k, v) for k, v in flattened.items()])
-            target_str = strutils.mask_password(target_str)
-            LOG.debug('RBAC: Policy Enforcement Target Data '
-                      '`%(action)s => target(%(target_str)s)`',
-                      {'action': action, 'target_str': target_str})
-
+        ctxt = cls._get_oslo_req_context()
         # Instantiate the enforcer object if needed.
         enforcer_obj = enforcer or cls()
         enforcer_obj._enforce(
-            credentials=creds, action=action, target=flattened)
+            credentials=ctxt, action=action, target=flattened)
         LOG.debug('RBAC: Authorization granted')
 
     @classmethod
