@@ -31,7 +31,6 @@ from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 from osprofiler import opts as profiler
 import osprofiler.sqlalchemy
-import six
 import sqlalchemy as sql
 from sqlalchemy.ext import declarative
 from sqlalchemy.orm.attributes import flag_modified, InstrumentedAttribute
@@ -103,8 +102,8 @@ def initialize_decorator(init):
                 if isinstance(attr, InstrumentedAttribute):
                     column = attr.property.columns[0]
                     if isinstance(column.type, String):
-                        if not isinstance(v, six.text_type):
-                            v = six.text_type(v)
+                        if not isinstance(v, str):
+                            v = str(v)
                         if column.type.length and column.type.length < len(v):
                             raise exception.StringLengthExceeded(
                                 string=v, type=k, length=column.type.length)
@@ -112,6 +111,7 @@ def initialize_decorator(init):
         init(self, *args, **kwargs)
     return initialize
 
+
 ModelBase.__init__ = initialize_decorator(ModelBase.__init__)
 
 
@@ -124,7 +124,9 @@ class JsonBlob(sql_types.TypeDecorator):
         return jsonutils.dumps(value)
 
     def process_result_value(self, value, dialect):
-        return jsonutils.loads(value)
+        if value is not None:
+            value = jsonutils.loads(value)
+        return value
 
 
 class DateTimeInt(sql_types.TypeDecorator):
@@ -519,11 +521,11 @@ def handle_conflicts(conflict_type='object'):
                 # exception details out with the raised Conflict exception
                 # as it can contain raw SQL.
                 LOG.debug(_conflict_msg, {'conflict_type': conflict_type,
-                                          'details': six.text_type(e)})
+                                          'details': e})
                 name = None
                 field = None
                 domain_id = None
-                # First element is unnessecary for extracting name and causes
+                # First element is unnecessary for extracting name and causes
                 # object not iterable error. Remove it.
                 params = args[1:]
                 # We want to store the duplicate objects name in the error
@@ -558,7 +560,7 @@ def handle_conflicts(conflict_type='object'):
                     # exception details out with the raised Conflict exception
                     # as it can contain raw SQL.
                     LOG.debug(_conflict_msg, {'conflict_type': conflict_type,
-                                              'details': six.text_type(e)})
+                                              'details': e})
                     # NOTE(morganfainberg): This is really a case where the SQL
                     # failed to store the data. This is not something that the
                     # user has done wrong. Example would be a ForeignKey is
