@@ -29,8 +29,8 @@ from oslo_serialization import jsonutils
 from oslo_utils import reflection
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import six
-from six import moves
+import urllib
+
 
 from keystone.common import password_hashing
 import keystone.conf
@@ -66,8 +66,6 @@ def resource_uuid(value):
         return value
     except ValueError:
         if len(value) <= 64:
-            if six.PY2 and isinstance(value, six.text_type):
-                value = value.encode('utf-8')
             return uuid.uuid5(RESOURCE_ID_NAMESPACE, value).hex
         raise ValueError(_('Length of transformable resource id > 64, '
                          'which is max allowed characters'))
@@ -94,14 +92,14 @@ class SmarterEncoder(jsonutils.json.JSONEncoder):
     """Help for JSON encoding dict-like objects."""
 
     def default(self, obj):
-        if not isinstance(obj, dict) and hasattr(obj, 'iteritems'):
-            return dict(obj.iteritems())
+        if not isinstance(obj, dict) and hasattr(obj, 'items'):
+            return dict(obj.items())
         return super(SmarterEncoder, self).default(obj)
 
 
 def hash_access_key(access):
     hash_ = hashlib.sha256()
-    if not isinstance(access, six.binary_type):
+    if not isinstance(access, bytes):
         access = access.encode('utf-8')
     hash_.update(access)
     return hash_.hexdigest()
@@ -119,23 +117,6 @@ def attr_as_boolean(val_attr):
     return strutils.bool_from_string(val_attr, default=True)
 
 
-def remove_duplicate_dicts_by_id(item_list):
-    """Return a list with duplicate items removed.
-
-    This compares duplicates based on item id. This is to account
-    for dictionaries of items that can contain values, such as a
-    list of tags, that will not be guaranteed to be in order.
-
-    :param item_list: a list of dictionaries
-    :returns: a list of unique dictionaries
-
-    """
-    unique = {}
-    for i in item_list:
-        unique[i['id']] = i
-    return list(unique.values())
-
-
 def auth_str_equal(provided, known):
     """Constant-time string comparison.
 
@@ -153,7 +134,7 @@ def auth_str_equal(provided, known):
     result = 0
     p_len = len(provided)
     k_len = len(known)
-    for i in moves.range(p_len):
+    for i in range(p_len):
         a = ord(provided[i]) if i < p_len else 0
         b = ord(known[i]) if i < k_len else 0
         result |= a ^ b
@@ -210,7 +191,7 @@ def get_unix_user(user=None):
     :returns: tuple of (uid, name)
 
     """
-    if isinstance(user, six.string_types):
+    if isinstance(user, str):
         try:
             user_info = pwd.getpwnam(user)
         except KeyError:
@@ -268,7 +249,7 @@ def get_unix_group(group=None):
     :returns: tuple of (gid, name)
 
     """
-    if isinstance(group, six.string_types):
+    if isinstance(group, str):
         try:
             group_info = grp.getgrnam(group)
         except KeyError:
@@ -391,15 +372,15 @@ def lower_case_hostname(url):
     # NOTE(gyee): according to
     # https://www.w3.org/TR/WD-html40-970708/htmlweb.html, the netloc portion
     # of the URL is case-insensitive
-    parsed = moves.urllib.parse.urlparse(url)
+    parsed = urllib.parse.urlparse(url)
     # Note: _replace method for named tuples is public and defined in docs
     replaced = parsed._replace(netloc=parsed.netloc.lower())
-    return moves.urllib.parse.urlunparse(replaced)
+    return urllib.parse.urlunparse(replaced)
 
 
 def remove_standard_port(url):
     # remove the default ports specified in RFC2616 and 2818
-    o = moves.urllib.parse.urlparse(url)
+    o = urllib.parse.urlparse(url)
     separator = ':'
     (host, separator, port) = o.netloc.partition(separator)
     if o.scheme.lower() == 'http' and port == '80':
@@ -411,7 +392,7 @@ def remove_standard_port(url):
     if o.scheme.lower() == 'https' and port == '443':
         o = o._replace(netloc=host)
 
-    return moves.urllib.parse.urlunparse(o)
+    return urllib.parse.urlunparse(o)
 
 
 def format_url(url, substitutions, silent_keyerror_failures=None):
@@ -473,4 +454,38 @@ def check_endpoint_url(url):
     try:
         url.replace('$(', '%(') % substitutions
     except (KeyError, TypeError, ValueError):
-        raise exception.URLValidationError(url)
+        raise exception.URLValidationError(url=url)
+
+
+def create_directory(directory, keystone_user_id=None, keystone_group_id=None):
+    """Attempt to create a directory if it doesn't exist.
+
+    :param directory: string containing the path of the directory to create.
+    :param keystone_user_id: the system ID of the process running keystone.
+    :param keystone_group_id: the system ID of the group running keystone.
+
+    """
+    if not os.access(directory, os.F_OK):
+        LOG.info(
+            '%s does not appear to exist; attempting to create it', directory
+        )
+
+        try:
+            os.makedirs(directory, 0o700)
+        except OSError:
+            LOG.error(
+                'Failed to create %s: either it already '
+                'exists or you don\'t have sufficient permissions to '
+                'create it', directory
+            )
+
+        if keystone_user_id and keystone_group_id:
+            os.chown(
+                directory,
+                keystone_user_id,
+                keystone_group_id)
+        elif keystone_user_id or keystone_group_id:
+            LOG.warning(
+                'Unable to change the ownership of key repository without '
+                'a keystone user ID and keystone group ID both being '
+                'provided: %s', directory)
