@@ -11,18 +11,15 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-from __future__ import absolute_import
 import uuid
 
 import ldap.filter
 from oslo_log import log
 from oslo_log import versionutils
-import six
 
-from keystone.common import driver_hints
 import keystone.conf
 from keystone import exception
-from keystone.i18n import _, _LW
+from keystone.i18n import _
 from keystone.identity.backends import base
 from keystone.identity.backends.ldap import common as common_ldap
 from keystone.identity.backends.ldap import models
@@ -31,12 +28,16 @@ from keystone.identity.backends.ldap import models
 CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
-_DEPRECATION_MSG = _('%s for the LDAP identity backend has been deprecated in '
-                     'the Mitaka release in favor of read-only identity LDAP '
-                     'access. It will be removed in the "O" release.')
+_DEPRECATION_MSG = ('%s for the LDAP identity backend has been deprecated in '
+                    'the Mitaka release in favor of read-only identity LDAP '
+                    'access. It will be removed in the "O" release.')
+
+READ_ONLY_LDAP_ERROR_MESSAGE = _("LDAP does not support write operations")
+
+LDAP_MATCHING_RULE_IN_CHAIN = "1.2.840.113556.1.4.1941"
 
 
-class Identity(base.IdentityDriverV8):
+class Identity(base.IdentityDriverBase):
     def __init__(self, conf=None):
         super(Identity, self).__init__()
         if conf is None:
@@ -83,23 +84,118 @@ class Identity(base.IdentityDriverV8):
     def list_users(self, hints):
         return self.user.get_all_filtered(hints)
 
+    def unset_default_project_id(self, project_id):
+        # This function is not implemented for the LDAP backend. The LDAP
+        # backend is readonly.
+        self._disallow_write()
+
     def get_user_by_name(self, user_name, domain_id):
         # domain_id will already have been handled in the Manager layer,
         # parameter left in so this matches the Driver specification
         return self.user.filter_attributes(self.user.get_by_name(user_name))
 
-    # CRUD
+    def get_group(self, group_id):
+        return self.group.get_filtered(group_id)
+
+    def get_group_by_name(self, group_name, domain_id):
+        # domain_id will already have been handled in the Manager layer,
+        # parameter left in so this matches the Driver specification
+        return self.group.get_filtered_by_name(group_name)
+
+    def list_groups_for_user(self, user_id, hints):
+        user_ref = self._get_user(user_id)
+        if self.conf.ldap.group_members_are_ids:
+            user_dn = user_ref['id']
+        else:
+            user_dn = user_ref['dn']
+        return self.group.list_user_groups_filtered(user_dn, hints)
+
+    def list_groups(self, hints):
+        return self.group.get_all_filtered(hints)
+
+    def _transform_group_member_ids(self, group_member_list):
+        for user_key in group_member_list:
+            if self.conf.ldap.group_members_are_ids:
+                user_id = user_key
+            else:
+                user_id = self.user._dn_to_id(user_key)
+            yield user_id
+
+    def list_users_in_group(self, group_id, hints):
+        users = []
+        group_members = self.group.list_group_users(group_id)
+        for user_id in self._transform_group_member_ids(group_members):
+            try:
+                users.append(self.user.get_filtered(user_id))
+            except exception.UserNotFound:
+                msg = ('Group member `%(user_id)s` for group `%(group_id)s`'
+                       ' not found in the directory. The user should be'
+                       ' removed from the group. The user will be ignored.')
+                LOG.debug(msg, dict(user_id=user_id, group_id=group_id))
+        return users
+
+    def check_user_in_group(self, user_id, group_id):
+        # Before doing anything, check that the user exists. This will raise
+        # a not found error if the user doesn't exist so we avoid doing extra
+        # work.
+        self.get_user(user_id)
+        member_list = self.group.list_group_users(group_id)
+        for group_member_id in self._transform_group_member_ids(member_list):
+            if group_member_id == user_id:
+                break
+        else:
+            raise exception.NotFound(_("User '%(user_id)s' not found in"
+                                       " group '%(group_id)s'") %
+                                     {'user_id': user_id,
+                                      'group_id': group_id})
+
+    # Unsupported methods
+    def _disallow_write(self):
+        if not common_ldap.WRITABLE:
+            raise exception.Forbidden(READ_ONLY_LDAP_ERROR_MESSAGE)
+
     def create_user(self, user_id, user):
+        self._disallow_write()
+        return self._create_user(user_id, user)
+
+    def update_user(self, user_id, user):
+        self._disallow_write()
+        return self._update_user(user_id, user)
+
+    def delete_user(self, user_id):
+        raise exception.Forbidden(READ_ONLY_LDAP_ERROR_MESSAGE)
+
+    def change_password(self, user_id, new_password):
+        raise exception.Forbidden(READ_ONLY_LDAP_ERROR_MESSAGE)
+
+    def add_user_to_group(self, user_id, group_id):
+        self._disallow_write()
+        self._add_user_to_group(user_id, group_id)
+
+    def remove_user_from_group(self, user_id, group_id):
+        raise exception.Forbidden(READ_ONLY_LDAP_ERROR_MESSAGE)
+
+    def create_group(self, group_id, group):
+        self._disallow_write()
+        return self._create_group(group_id, group)
+
+    def update_group(self, group_id, group):
+        self._disallow_write()
+        return self._update_group(group_id, group)
+
+    def delete_group(self, group_id):
+        raise exception.Forbidden(READ_ONLY_LDAP_ERROR_MESSAGE)
+
+    # Test implementations
+    def _create_user(self, user_id, user):
         msg = _DEPRECATION_MSG % "create_user"
         versionutils.report_deprecated_feature(LOG, msg)
-        self.user.check_allow_create()
         user_ref = self.user.create(user)
         return self.user.filter_attributes(user_ref)
 
-    def update_user(self, user_id, user):
+    def _update_user(self, user_id, user):
         msg = _DEPRECATION_MSG % "update_user"
         versionutils.report_deprecated_feature(LOG, msg)
-        self.user.check_allow_update()
         old_obj = self.user.get(user_id)
         if 'name' in user and old_obj.get('name') != user['name']:
             raise exception.Conflict(_('Cannot change user name'))
@@ -116,115 +212,23 @@ class Identity(base.IdentityDriverV8):
         self.user.update(user_id, user, old_obj)
         return self.user.get_filtered(user_id)
 
-    def change_password(self, user_id, new_password):
-        raise exception.NotImplemented(
-            _('Self-service user password changes are not implemented for '
-              'LDAP.'))
-
-    def delete_user(self, user_id):
-        msg = _DEPRECATION_MSG % "delete_user"
-        versionutils.report_deprecated_feature(LOG, msg)
-        self.user.check_allow_delete()
-        user = self.user.get(user_id)
-        user_dn = user['dn']
-        groups = self.group.list_user_groups(user_dn)
-        for group in groups:
-            group_ref = self.group.get(group['id'], '*')  # unfiltered
-            group_dn = group_ref['dn']
-            try:
-                super(GroupApi, self.group).remove_member(user_dn, group_dn)
-            except ldap.NO_SUCH_ATTRIBUTE:
-                LOG.warning(
-                    _LW('User %(user)s was not removed from group %(group)s '
-                        'because the relationship was not found'),
-                    {'user': user_id, 'group': group['id']})
-
-        if hasattr(user, 'tenant_id'):
-            self.project.remove_user(user.tenant_id, user_dn)
-        self.user.delete(user_id)
-
-    def create_group(self, group_id, group):
+    def _create_group(self, group_id, group):
         msg = _DEPRECATION_MSG % "create_group"
         versionutils.report_deprecated_feature(LOG, msg)
-        self.group.check_allow_create()
         return common_ldap.filter_entity(self.group.create(group))
 
-    def get_group(self, group_id):
-        return self.group.get_filtered(group_id)
-
-    def get_group_by_name(self, group_name, domain_id):
-        # domain_id will already have been handled in the Manager layer,
-        # parameter left in so this matches the Driver specification
-        return self.group.get_filtered_by_name(group_name)
-
-    def update_group(self, group_id, group):
+    def _update_group(self, group_id, group):
         msg = _DEPRECATION_MSG % "update_group"
         versionutils.report_deprecated_feature(LOG, msg)
-        self.group.check_allow_update()
         return common_ldap.filter_entity(self.group.update(group_id, group))
 
-    def delete_group(self, group_id):
-        msg = _DEPRECATION_MSG % "delete_group"
-        versionutils.report_deprecated_feature(LOG, msg)
-        self.group.check_allow_delete()
-        return self.group.delete(group_id)
-
-    def add_user_to_group(self, user_id, group_id):
+    def _add_user_to_group(self, user_id, group_id):
         msg = _DEPRECATION_MSG % "add_user_to_group"
         versionutils.report_deprecated_feature(LOG, msg)
         user_ref = self._get_user(user_id)
         user_dn = user_ref['dn']
         self.group.add_user(user_dn, group_id, user_id)
 
-    def remove_user_from_group(self, user_id, group_id):
-        msg = _DEPRECATION_MSG % "remove_user_from_group"
-        versionutils.report_deprecated_feature(LOG, msg)
-        user_ref = self._get_user(user_id)
-        user_dn = user_ref['dn']
-        self.group.remove_user(user_dn, group_id, user_id)
-
-    def list_groups_for_user(self, user_id, hints):
-        user_ref = self._get_user(user_id)
-        if self.conf.ldap.group_members_are_ids:
-            user_dn = user_ref['id']
-        else:
-            user_dn = user_ref['dn']
-        return self.group.list_user_groups_filtered(user_dn, hints)
-
-    def list_groups(self, hints):
-        return self.group.get_all_filtered(hints)
-
-    def list_users_in_group(self, group_id, hints):
-        users = []
-        for user_key in self.group.list_group_users(group_id):
-            if self.conf.ldap.group_members_are_ids:
-                user_id = user_key
-            else:
-                user_id = self.user._dn_to_id(user_key)
-
-            try:
-                users.append(self.user.get_filtered(user_id))
-            except exception.UserNotFound:
-                LOG.debug(("Group member '%(user_key)s' not found in"
-                           " '%(group_id)s'. The user should be removed"
-                           " from the group. The user will be ignored."),
-                          dict(user_key=user_key, group_id=group_id))
-        return users
-
-    def check_user_in_group(self, user_id, group_id):
-        user_refs = self.list_users_in_group(group_id, driver_hints.Hints())
-        for x in user_refs:
-            if x['id'] == user_id:
-                break
-        else:
-            # Try to fetch the user to see if it even exists.  This
-            # will raise a more accurate exception.
-            self.get_user(user_id)
-            raise exception.NotFound(_("User '%(user_id)s' not found in"
-                                       " group '%(group_id)s'") %
-                                     {'user_id': user_id,
-                                      'group_id': group_id})
-
 
 # TODO(termie): turn this into a data object and move logic to driver
 class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
@@ -261,7 +265,7 @@ class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
             # This could be a bool or a string.  If it's a string,
             # we need to convert it so we can invert it properly.
             enabled = obj.get('enabled', self.enabled_default)
-            if isinstance(enabled, six.string_types):
+            if isinstance(enabled, str):
                 if enabled.lower() == 'true':
                     enabled = True
                 else:
@@ -281,6 +285,8 @@ class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
         del values['enabled_nomask']
 
     def create(self, values):
+        if 'options' in values:
+            values.pop('options')  # can't specify options
         if self.enabled_mask:
             orig_enabled = values['enabled']
             self.mask_enabled_attribute(values)
@@ -294,11 +300,27 @@ class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
         if self.enabled_mask or (self.enabled_invert and
                                  not self.enabled_emulation):
             values['enabled'] = orig_enabled
+        values['options'] = {}  # options always empty
         return values
 
+    def get(self, user_id, ldap_filter=None):
+        obj = super(UserApi, self).get(user_id, ldap_filter=ldap_filter)
+        obj['options'] = {}  # options always empty
+        return obj
+
     def get_filtered(self, user_id):
-        user = self.get(user_id)
-        return self.filter_attributes(user)
+        try:
+            user = self.get(user_id)
+            return self.filter_attributes(user)
+        except ldap.NO_SUCH_OBJECT:
+            raise self.NotFound(user_id=user_id)
+
+    def get_all(self, ldap_filter=None, hints=None):
+        objs = super(UserApi, self).get_all(ldap_filter=ldap_filter,
+                                            hints=hints)
+        for obj in objs:
+            obj['options'] = {}  # options always empty
+        return objs
 
     def get_all_filtered(self, hints):
         query = self.filter_query(hints, self.ldap_filter)
@@ -317,6 +339,18 @@ class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
 
         return common_ldap.dn_startswith(dn, self.tree_dn)
 
+    def update(self, user_id, values, old_obj=None):
+        if old_obj is None:
+            old_obj = self.get(user_id)
+        # don't support updating options
+        if 'options' in old_obj:
+            old_obj.pop('options')
+        if 'options' in values:
+            values.pop('options')
+        values = super(UserApi, self).update(user_id, values, old_obj)
+        values['options'] = {}  # options always empty
+        return values
+
 
 class GroupApi(common_ldap.BaseLdap):
     DEFAULT_OU = 'ou=UserGroups'
@@ -338,6 +372,7 @@ class GroupApi(common_ldap.BaseLdap):
 
     def __init__(self, conf):
         super(GroupApi, self).__init__(conf)
+        self.group_ad_nesting = conf.ldap.group_ad_nesting
         self.member_attribute = (conf.ldap.group_member_attribute
                                  or self.DEFAULT_MEMBER_ATTRIBUTE)
 
@@ -349,19 +384,6 @@ class GroupApi(common_ldap.BaseLdap):
             data.pop('description')
         return super(GroupApi, self).create(data)
 
-    def delete(self, group_id):
-        if self.subtree_delete_enabled:
-            super(GroupApi, self).delete_tree(group_id)
-        else:
-            # TODO(spzala): this is only placeholder for group and domain
-            # role support which will be added under bug 1101287
-
-            group_ref = self.get(group_id)
-            group_dn = group_ref['dn']
-            if group_dn:
-                self._delete_tree_nodes(group_dn, ldap.SCOPE_ONELEVEL)
-            super(GroupApi, self).delete(group_id)
-
     def update(self, group_id, values):
         old_obj = self.get(group_id)
         return super(GroupApi, self).update(group_id, values, old_obj)
@@ -376,26 +398,32 @@ class GroupApi(common_ldap.BaseLdap):
                 'User %(user_id)s is already a member of group %(group_id)s') %
                 {'user_id': user_id, 'group_id': group_id})
 
-    def remove_user(self, user_dn, group_id, user_id):
-        group_ref = self.get(group_id)
-        group_dn = group_ref['dn']
-        try:
-            super(GroupApi, self).remove_member(user_dn, group_dn)
-        except ldap.NO_SUCH_ATTRIBUTE:
-            raise exception.UserNotFound(user_id=user_id)
-
     def list_user_groups(self, user_dn):
         """Return a list of groups for which the user is a member."""
         user_dn_esc = ldap.filter.escape_filter_chars(user_dn)
-        query = '(%s=%s)' % (self.member_attribute,
-                             user_dn_esc)
+        if self.group_ad_nesting:
+            query = '(%s:%s:=%s)' % (
+                self.member_attribute,
+                LDAP_MATCHING_RULE_IN_CHAIN,
+                user_dn_esc)
+        else:
+            query = '(%s=%s)' % (self.member_attribute,
+                                 user_dn_esc)
         return self.get_all(query)
 
     def list_user_groups_filtered(self, user_dn, hints):
         """Return a filtered list of groups for which the user is a member."""
         user_dn_esc = ldap.filter.escape_filter_chars(user_dn)
-        query = '(%s=%s)' % (self.member_attribute,
-                             user_dn_esc)
+        if self.group_ad_nesting:
+            # Hardcoded to member as that is how the Matching Rule in Chain
+            # Mechanisms expects it.  The member_attribute might actually be
+            # member_of elsewhere, so they are not the same.
+            query = '(member:%s:=%s)' % (
+                LDAP_MATCHING_RULE_IN_CHAIN,
+                user_dn_esc)
+        else:
+            query = '(%s=%s)' % (self.member_attribute,
+                                 user_dn_esc)
         return self.get_all_filtered(hints, query)
 
     def list_group_users(self, group_id):
@@ -404,8 +432,20 @@ class GroupApi(common_ldap.BaseLdap):
         group_dn = group_ref['dn']
 
         try:
-            attrs = self._ldap_get_list(group_dn, ldap.SCOPE_BASE,
-                                        attrlist=[self.member_attribute])
+            if self.group_ad_nesting:
+                # NOTE(ayoung): LDAP_SCOPE is used here instead of hard-
+                # coding to SCOPE_SUBTREE to get through the unit tests.
+                # However, it is also probably more correct.
+                attrs = self._ldap_get_list(
+                    self.tree_dn, self.LDAP_SCOPE,
+                    query_params={
+                        "member:%s:" % LDAP_MATCHING_RULE_IN_CHAIN:
+                        group_dn},
+                    attrlist=[self.member_attribute])
+            else:
+                attrs = self._ldap_get_list(group_dn, ldap.SCOPE_BASE,
+                                            attrlist=[self.member_attribute])
+
         except ldap.NO_SUCH_OBJECT:
             raise self.NotFound(group_id=group_id)
 
@@ -413,8 +453,6 @@ class GroupApi(common_ldap.BaseLdap):
         for dn, member in attrs:
             user_dns = member.get(self.member_attribute, [])
             for user_dn in user_dns:
-                if self._is_dumb_member(user_dn):
-                    continue
                 users.append(user_dn)
         return users
 
