@@ -48,19 +48,24 @@ class User(sql.ModelBase, sql.ModelDictMixinWithExtras):
         backref='user',
         collection_class=collections.attribute_mapped_collection('option_id'))
     local_user = orm.relationship('LocalUser', uselist=False,
-                                  single_parent=True, lazy='subquery',
+                                  single_parent=True, lazy='joined',
                                   cascade='all,delete-orphan', backref='user')
     federated_users = orm.relationship('FederatedUser',
                                        single_parent=True,
-                                       lazy='subquery',
+                                       lazy='joined',
                                        cascade='all,delete-orphan',
                                        backref='user')
     nonlocal_user = orm.relationship('NonLocalUser',
                                      uselist=False,
                                      single_parent=True,
-                                     lazy='subquery',
+                                     lazy='joined',
                                      cascade='all,delete-orphan',
                                      backref='user')
+    expiring_user_group_memberships = orm.relationship(
+        'ExpiringUserGroupMembership',
+        cascade='all, delete-orphan',
+        backref="user"
+    )
     created_at = sql.Column(sql.DateTime, nullable=True)
     last_active_at = sql.Column(sql.Date, nullable=True)
     # unique constraint needed here to support composite fk constraints
@@ -82,9 +87,13 @@ class User(sql.ModelBase, sql.ModelDictMixinWithExtras):
 
     @name.setter
     def name(self, value):
-        if not self.local_user:
+        if self.federated_users:
+            self.federated_users[0].display_name = value
+        elif self.local_user:
+            self.local_user.name = value
+        else:
             self.local_user = LocalUser()
-        self.local_user.name = value
+            self.local_user.name = value
 
     @name.expression
     def name(cls):
@@ -105,10 +114,7 @@ class User(sql.ModelBase, sql.ModelDictMixinWithExtras):
     def password(self):
         """Return the current password."""
         if self.password_ref:
-            if self.password_ref.password_hash is not None:
-                return self.password_ref.password_hash
-            else:
-                return self.password_ref.password
+            return self.password_ref.password_hash
         return None
 
     @property
@@ -140,6 +146,7 @@ class User(sql.ModelBase, sql.ModelDictMixinWithExtras):
         # truncate extra passwords
         if self.local_user.passwords:
             unique_cnt = CONF.security_compliance.unique_last_password_count
+            unique_cnt = unique_cnt + 1 if unique_cnt == 0 else unique_cnt
             self.local_user.passwords = self.local_user.passwords[-unique_cnt:]
         # set all previous passwords to be expired
         for ref in self.local_user.passwords:
@@ -148,19 +155,17 @@ class User(sql.ModelBase, sql.ModelDictMixinWithExtras):
         new_password_ref = Password()
 
         hashed_passwd = None
-        hashed_compat = None
         if value is not None:
             # NOTE(notmorgan): hash the passwords, never directly bind the
-            # "value" in the unhashed form to hashed_passwd or hashed_compat
-            # to ensure the unhashed password cannot end up in the db. If an
-            # unhashed password ends up in the DB, it cannot be used for auth,
-            # it is however incorrect and could leak user credentials (due to
-            # users doing insecure things such as sharing passwords across
+            # "value" in the unhashed form to hashed_passwd to ensure the
+            # unhashed password cannot end up in the db. If an unhashed
+            # password ends up in the DB, it cannot be used for auth, it is
+            # however incorrect and could leak user credentials (due to users
+            # doing insecure things such as sharing passwords across
             # different systems) to unauthorized parties.
             hashed_passwd = password_hashing.hash_password(value)
 
         new_password_ref.password_hash = hashed_passwd
-        new_password_ref.password = hashed_compat
         new_password_ref.created_at = now
         new_password_ref.expires_at = self._get_password_expires_at(now)
         self.local_user.passwords.append(new_password_ref)
@@ -184,7 +189,7 @@ class User(sql.ModelBase, sql.ModelDictMixinWithExtras):
 
     @password.expression
     def password(cls):
-        return Password.password
+        return Password.password_hash
 
     # NOTE(stevemar): we use a hybrid property here because we leverage the
     # expression method, see `@enabled.expression` and `User._enabled` below.
@@ -194,13 +199,18 @@ class User(sql.ModelBase, sql.ModelDictMixinWithExtras):
         if self._enabled:
             max_days = (
                 CONF.security_compliance.disable_user_account_days_inactive)
+            inactivity_exempt = getattr(
+                self.get_resource_option(
+                    iro.IGNORE_USER_INACTIVITY_OPT.option_id),
+                'option_value',
+                False)
             last_active = self.last_active_at
             if not last_active and self.created_at:
                 last_active = self.created_at.date()
             if max_days and last_active:
                 now = datetime.datetime.utcnow().date()
                 days_inactive = (now - last_active).days
-                if days_inactive >= max_days:
+                if days_inactive >= max_days and not inactivity_exempt:
                     self._enabled = False
         return self._enabled
 
@@ -272,7 +282,7 @@ class LocalUser(sql.ModelBase, sql.ModelDictMixin):
     passwords = orm.relationship('Password',
                                  single_parent=True,
                                  cascade='all,delete-orphan',
-                                 lazy='subquery',
+                                 lazy='joined',
                                  backref='local_user',
                                  order_by='Password.created_at_int')
     failed_auth_count = sql.Column(sql.Integer, nullable=True)
@@ -288,15 +298,11 @@ class LocalUser(sql.ModelBase, sql.ModelDictMixin):
 
 class Password(sql.ModelBase, sql.ModelDictMixin):
     __tablename__ = 'password'
-    attributes = ['id', 'local_user_id', 'password', 'password_hash',
-                  'created_at', 'expires_at']
+    attributes = ['id', 'local_user_id', 'password_hash', 'created_at',
+                  'expires_at']
     id = sql.Column(sql.Integer, primary_key=True)
     local_user_id = sql.Column(sql.Integer, sql.ForeignKey('local_user.id',
                                ondelete='CASCADE'))
-    # TODO(notmorgan): in the Q release the "password" field can be dropped as
-    # long as data migration exists to move the hashes over to the
-    # password_hash column if no value is in the password_hash column.
-    password = sql.Column(sql.String(128), nullable=True)
     password_hash = sql.Column(sql.String(255), nullable=True)
 
     # TODO(lbragstad): Once Rocky opens for development, the _created_at and
@@ -350,7 +356,8 @@ class FederatedUser(sql.ModelBase, sql.ModelDictMixin):
         sql.UniqueConstraint('idp_id', 'protocol_id', 'unique_id'),
         sqlalchemy.ForeignKeyConstraint(['protocol_id', 'idp_id'],
                                         ['federation_protocol.id',
-                                         'federation_protocol.idp_id'])
+                                         'federation_protocol.idp_id'],
+                                        ondelete='CASCADE')
     )
 
 
@@ -377,6 +384,11 @@ class Group(sql.ModelBase, sql.ModelDictMixinWithExtras):
     domain_id = sql.Column(sql.String(64), nullable=False)
     description = sql.Column(sql.Text())
     extra = sql.Column(sql.JsonBlob())
+    expiring_user_group_memberships = orm.relationship(
+        'ExpiringUserGroupMembership',
+        cascade='all, delete-orphan',
+        backref="group"
+    )
     # Unique constraint across two columns to create the separation
     # rather than just only 'name' being unique
     __table_args__ = (sql.UniqueConstraint('domain_id', 'name'),)
@@ -394,6 +406,34 @@ class UserGroupMembership(sql.ModelBase, sql.ModelDictMixin):
                           primary_key=True)
 
 
+class ExpiringUserGroupMembership(sql.ModelBase, sql.ModelDictMixin):
+    """Expiring group membership through federation mapping rules."""
+
+    __tablename__ = 'expiring_user_group_membership'
+    user_id = sql.Column(sql.String(64),
+                         sql.ForeignKey('user.id'),
+                         primary_key=True)
+    group_id = sql.Column(sql.String(64),
+                          sql.ForeignKey('group.id'),
+                          primary_key=True)
+    idp_id = sql.Column(sql.String(64),
+                        sql.ForeignKey('identity_provider.id',
+                                       ondelete='CASCADE'),
+                        primary_key=True)
+    last_verified = sql.Column(sql.DateTime, nullable=False)
+
+    @hybrid_property
+    def expires(self):
+        ttl = self.idp.authorization_ttl
+        if not ttl:
+            ttl = CONF.federation.default_authorization_ttl
+        return self.last_verified + datetime.timedelta(minutes=ttl)
+
+    @hybrid_property
+    def expired(self):
+        return self.expires <= datetime.datetime.utcnow()
+
+
 class UserOption(sql.ModelBase):
     __tablename__ = 'user_option'
     user_id = sql.Column(sql.String(64), sql.ForeignKey('user.id',
