@@ -28,7 +28,6 @@ from pycadf import reason
 
 from keystone import assignment  # TODO(lbragstad): Decouple this dependency
 from keystone.common import cache
-from keystone.common import clean
 from keystone.common import driver_hints
 from keystone.common import manager
 from keystone.common import provider_api
@@ -111,9 +110,8 @@ class DomainConfigs(provider_api.ProviderAPIMixin, dict):
         try:
             domain_ref = resource_api.get_domain_by_name(domain_name)
         except exception.DomainNotFound:
-            LOG.warning(
-                ('Invalid domain name (%s) found in config file name'),
-                domain_name)
+            LOG.warning('Invalid domain name (%s) found in config file name',
+                        domain_name)
             return
 
         # Create a new entry in the domain config dict, which contains
@@ -502,20 +500,21 @@ class Manager(manager.Manager):
 
         driver = self._select_identity_driver(domain_id)
 
-        user_refs = self.list_users(domain_scope=domain_id)
-        group_refs = self.list_groups(domain_scope=domain_id)
-
-        for group in group_refs:
-            # Cleanup any existing groups.
-            try:
-                self.delete_group(group['id'])
-            except exception.GroupNotFound:
-                LOG.debug(('Group %(groupid)s not found when deleting domain '
-                           'contents for %(domainid)s, continuing with '
-                           'cleanup.'),
-                          {'groupid': group['id'], 'domainid': domain_id})
+        if driver.is_sql:
+            group_refs = self.list_groups(domain_scope=domain_id)
+            for group in group_refs:
+                # Cleanup any existing groups.
+                try:
+                    self.delete_group(group['id'])
+                except exception.GroupNotFound:
+                    LOG.debug(('Group %(groupid)s not found when deleting '
+                               'domain contents for %(domainid)s, continuing '
+                               'with cleanup.'),
+                              {'groupid': group['id'], 'domainid': domain_id})
 
         # And finally, delete the users themselves
+        user_refs = self.list_users(domain_scope=domain_id)
+
         for user in user_refs:
             try:
                 if not driver.is_sql:
@@ -744,13 +743,13 @@ class Manager(manager.Manager):
         if (not driver.is_domain_aware() and driver == self.driver and
             domain_id != CONF.identity.default_domain_id and
                 domain_id is not None):
-                    LOG.warning('Found multiple domains being mapped to a '
-                                'driver that does not support that (e.g. '
-                                'LDAP) - Domain ID: %(domain)s, '
-                                'Default Driver: %(driver)s',
-                                {'domain': domain_id,
-                                 'driver': (driver == self.driver)})
-                    raise exception.DomainNotFound(domain_id=domain_id)
+            LOG.warning('Found multiple domains being mapped to a '
+                        'driver that does not support that (e.g. '
+                        'LDAP) - Domain ID: %(domain)s, '
+                        'Default Driver: %(driver)s',
+                        {'domain': domain_id,
+                         'driver': (driver == self.driver)})
+            raise exception.DomainNotFound(domain_id=domain_id)
         return driver
 
     def _get_domain_driver_and_entity_id(self, public_id):
@@ -895,11 +894,10 @@ class Manager(manager.Manager):
     # - select the right driver for this domain
     # - clear/set domain_ids for drivers that do not support domains
     # - create any ID mapping that might be required
-
     @notifications.emit_event('authenticate')
     @domains_configured
     @exception_translated('assertion')
-    def authenticate(self, request, user_id, password):
+    def authenticate(self, user_id, password):
         domain_id, driver, entity_id = (
             self._get_domain_driver_and_entity_id(user_id))
         ref = driver.authenticate(entity_id, password)
@@ -926,15 +924,65 @@ class Manager(manager.Manager):
                 # backward compatible
                 pass
 
+    def _validate_federated_objects(self, fed_obj_list):
+        # Validate that the ipd and protocols exist
+        for fed_obj in fed_obj_list:
+            try:
+                self.federation_api.get_idp(fed_obj['idp_id'])
+            except exception.IdentityProviderNotFound:
+                msg = (_("Could not find Identity Provider: %s")
+                       % fed_obj['idp_id'])
+                raise exception.ValidationError(msg)
+            for protocol in fed_obj['protocols']:
+                try:
+                    self.federation_api.get_protocol(fed_obj['idp_id'],
+                                                     protocol['protocol_id'])
+                except exception.FederatedProtocolNotFound:
+                    msg = (_("Could not find federated protocol "
+                             "%(protocol)s for Identity Provider: %(idp)s.")
+                           % {'protocol': protocol['protocol_id'],
+                              'idp': fed_obj['idp_id']})
+                    raise exception.ValidationError(msg)
+
+    def _create_federated_objects(self, user_ref, fed_obj_list):
+        for fed_obj in fed_obj_list:
+            for protocols in fed_obj['protocols']:
+                federated_dict = {
+                    'user_id': user_ref['id'],
+                    'idp_id': fed_obj['idp_id'],
+                    'protocol_id': protocols['protocol_id'],
+                    'unique_id': protocols['unique_id'],
+                    'display_name': user_ref['name']
+                }
+                self.shadow_users_api.create_federated_object(
+                    federated_dict)
+
+    def _create_user_with_federated_objects(self, user, driver):
+        # If the user did not pass a federated object along inside the user
+        # object then we simply create the user as normal.
+        if not user.get('federated'):
+            if 'federated' in user:
+                del user['federated']
+            user = driver.create_user(user['id'], user)
+            return user
+        # Otherwise, validate the federated object and create the user.
+        else:
+            user_ref = user.copy()
+            del user['federated']
+            self._validate_federated_objects(user_ref['federated'])
+            user = driver.create_user(user['id'], user)
+            self._create_federated_objects(user_ref, user_ref['federated'])
+            user['federated'] = user_ref['federated']
+            return user
+
     @domains_configured
     @exception_translated('user')
     def create_user(self, user_ref, initiator=None):
         user = user_ref.copy()
         if 'password' in user:
             validators.validate_password(user['password'])
-        user['name'] = clean.user_name(user['name'])
+        user['name'] = user['name'].strip()
         user.setdefault('enabled', True)
-        user['enabled'] = clean.user_enabled(user['enabled'])
         domain_id = user['domain_id']
         PROVIDERS.resource_api.get_domain(domain_id)
 
@@ -949,7 +997,7 @@ class Manager(manager.Manager):
         # the underlying driver so that it could conform to rules set down by
         # that particular driver type.
         user['id'] = uuid.uuid4().hex
-        ref = driver.create_user(user['id'], user)
+        ref = self._create_user_with_federated_objects(user, driver)
         notifications.Audit.created(self._USER, user['id'], initiator)
         return self._set_domain_id_and_mapping(
             ref, domain_id, driver, mapping.EntityType.USER)
@@ -961,6 +1009,10 @@ class Manager(manager.Manager):
         domain_id, driver, entity_id = (
             self._get_domain_driver_and_entity_id(user_id))
         ref = driver.get_user(entity_id)
+        # Add user's federated objects
+        fed_objects = self.shadow_users_api.get_federated_objects(user_id)
+        if fed_objects:
+            ref['federated'] = fed_objects
         return self._set_domain_id_and_mapping(
             ref, domain_id, driver, mapping.EntityType.USER)
 
@@ -1020,7 +1072,7 @@ class Manager(manager.Manager):
                 try:
                     filter_['comparator'] = operators[op]
                 except KeyError:
-                    raise exception.InvalidOperatorError(op)
+                    raise exception.InvalidOperatorError(_op=op)
         return hints
 
     def _handle_shadow_and_local_users(self, driver, hints):
@@ -1073,6 +1125,32 @@ class Manager(manager.Manager):
             if new_ref['domain_id'] != orig_ref['domain_id']:
                 raise exception.ValidationError(_('Cannot change Domain ID'))
 
+    def _update_user_with_federated_objects(self, user, driver, entity_id):
+        # If the user did not pass a federated object along inside the user
+        # object then we simply update the user as normal and add the
+        # currently associated federated objects to user to be added to the
+        # dictionary.
+        if not user.get('federated'):
+            if 'federated' in user:
+                del user['federated']
+            user = driver.update_user(entity_id, user)
+            fed_objects = self.shadow_users_api.get_federated_objects(
+                user['id'])
+            if fed_objects:
+                user['federated'] = fed_objects
+            return user
+        # Otherwise, we validate, remove the previous user's federated objects,
+        # and update the user along with their updated federated objects.
+        else:
+            user_ref = user.copy()
+            self._validate_federated_objects(user_ref['federated'])
+            self.shadow_users_api.delete_federated_object(entity_id)
+            del user['federated']
+            user = driver.update_user(entity_id, user)
+            self._create_federated_objects(user, user_ref['federated'])
+            user['federated'] = user_ref['federated']
+            return user
+
     @domains_configured
     @exception_translated('user')
     def update_user(self, user_id, user_ref, initiator=None):
@@ -1082,9 +1160,7 @@ class Manager(manager.Manager):
         if 'password' in user:
             validators.validate_password(user['password'])
         if 'name' in user:
-            user['name'] = clean.user_name(user['name'])
-        if 'enabled' in user:
-            user['enabled'] = clean.user_enabled(user['enabled'])
+            user['name'] = user['name'].strip()
         if 'id' in user:
             if user_id != user['id']:
                 raise exception.ValidationError(_('Cannot change user ID'))
@@ -1103,7 +1179,7 @@ class Manager(manager.Manager):
         self.get_user_by_name.invalidate(self, old_user_ref['name'],
                                          old_user_ref['domain_id'])
 
-        ref = driver.update_user(entity_id, user)
+        ref = self._update_user_with_federated_objects(user, driver, entity_id)
 
         notifications.Audit.updated(self._USER, user_id, initiator)
 
@@ -1140,7 +1216,7 @@ class Manager(manager.Manager):
         self.get_user_by_name.invalidate(self, user_old['name'],
                                          user_old['domain_id'])
         for fed_user in fed_users:
-            self.shadow_federated_user.invalidate(
+            self._shadow_federated_user.invalidate(
                 self, fed_user['idp_id'], fed_user['protocol_id'],
                 fed_user['unique_id'], fed_user['display_name'],
                 user_old.get('extra', {}).get('email'))
@@ -1169,7 +1245,7 @@ class Manager(manager.Manager):
         # the underlying driver so that it could conform to rules set down by
         # that particular driver type.
         group['id'] = uuid.uuid4().hex
-        group['name'] = clean.group_name(group['name'])
+        group['name'] = group['name'].strip()
         ref = driver.create_group(group['id'], group)
 
         notifications.Audit.created(self._GROUP, group['id'], initiator)
@@ -1204,7 +1280,7 @@ class Manager(manager.Manager):
             self._get_domain_driver_and_entity_id(group_id))
         group = self._clear_domain_id_if_domain_unaware(driver, group)
         if 'name' in group:
-            group['name'] = clean.group_name(group['name'])
+            group['name'] = group['name'].strip()
         ref = driver.update_group(entity_id, group)
         self.get_group.invalidate(self, group_id)
         notifications.Audit.updated(self._GROUP, group_id, initiator)
@@ -1314,6 +1390,9 @@ class Manager(manager.Manager):
             # driver selection, so remove any such filter
             self._mark_domain_id_filter_satisfied(hints)
         ref_list = driver.list_groups_for_user(entity_id, hints)
+        for ref in ref_list:
+            if 'membership_expires_at' not in ref:
+                ref['membership_expires_at'] = None
         return self._set_domain_id_and_mapping(
             ref_list, domain_id, driver, mapping.EntityType.GROUP)
 
@@ -1372,12 +1451,12 @@ class Manager(manager.Manager):
                                                 group_entity_id)
 
     @domains_configured
-    def change_password(self, request, user_id, original_password,
+    def change_password(self, user_id, original_password,
                         new_password, initiator=None):
 
         # authenticate() will raise an AssertionError if authentication fails
         try:
-            self.authenticate(request, user_id, original_password)
+            self.authenticate(user_id, original_password)
         except exception.PasswordExpired:
             # If a password has expired, we want users to be able to change it
             pass
@@ -1404,18 +1483,8 @@ class Manager(manager.Manager):
             return PROVIDERS.shadow_users_api.create_nonlocal_user(user)
 
     @MEMOIZE
-    def shadow_federated_user(self, idp_id, protocol_id, unique_id,
-                              display_name, email=None):
-        """Map a federated user to a user.
-
-        :param idp_id: identity provider id
-        :param protocol_id: protocol id
-        :param unique_id: unique id for the user within the IdP
-        :param display_name: user's display name
-        :param email: user's email
-
-        :returns: dictionary of the mapped User entity
-        """
+    def _shadow_federated_user(self, idp_id, protocol_id, unique_id,
+                               display_name, email=None):
         user_dict = {}
         try:
             PROVIDERS.shadow_users_api.update_federated_user_display_name(
@@ -1442,6 +1511,29 @@ class Manager(manager.Manager):
         PROVIDERS.shadow_users_api.set_last_active_at(user_dict['id'])
         return user_dict
 
+    def shadow_federated_user(self, idp_id, protocol_id, unique_id,
+                              display_name, email=None, group_ids=None):
+        """Map a federated user to a user.
+
+        :param idp_id: identity provider id
+        :param protocol_id: protocol id
+        :param unique_id: unique id for the user within the IdP
+        :param display_name: user's display name
+        :param email: user's email
+        :param group_ids: list of group ids to add the user to
+
+        :returns: dictionary of the mapped User entity
+        """
+        user_dict = self._shadow_federated_user(
+            idp_id, protocol_id, unique_id, display_name, email)
+        # Note(knikolla): The shadowing operation can be cached,
+        # however we need to update the expiring group memberships.
+        if group_ids:
+            for group_id in group_ids:
+                PROVIDERS.shadow_users_api.add_user_to_group_expires(
+                    user_dict['id'], group_id)
+        return user_dict
+
 
 class MappingManager(manager.Manager):
     """Default pivot point for the ID Mapping backend."""
