@@ -13,11 +13,11 @@
 import copy
 import datetime
 import sqlalchemy
-import uuid
 
 from oslo_config import cfg
 from oslo_db import api as oslo_db_api
 
+from keystone.common import provider_api
 from keystone.common import sql
 from keystone import exception
 from keystone.identity.backends import base as identity_base
@@ -26,13 +26,21 @@ from keystone.identity.shadow_backends import base
 
 
 CONF = cfg.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class ShadowUsers(base.ShadowUsersDriverBase):
     @sql.handle_conflicts(conflict_type='federated_user')
     def create_federated_user(self, domain_id, federated_dict, email=None):
+
+        local_entity = {'domain_id': domain_id,
+                        'local_id': federated_dict['unique_id'],
+                        'entity_type': 'user'}
+
+        public_id = PROVIDERS.id_generator_api.generate_public_ID(local_entity)
+
         user = {
-            'id': uuid.uuid4().hex,
+            'id': public_id,
             'domain_id': domain_id,
             'enabled': True
         }
@@ -46,6 +54,31 @@ class ShadowUsers(base.ShadowUsersDriverBase):
             session.add(user_ref)
             return identity_base.filter_user(user_ref.to_dict())
 
+    @sql.handle_conflicts(conflict_type='federated_user')
+    def create_federated_object(self, fed_dict):
+        with sql.session_for_write() as session:
+            fed_ref = model.FederatedUser.from_dict(fed_dict)
+            session.add(fed_ref)
+
+    def delete_federated_object(self, user_id):
+        with sql.session_for_write() as session:
+            q = session.query(model.FederatedUser)
+            q = q.filter(model.FederatedUser.user_id == user_id)
+            q.delete(False)
+
+    def get_federated_objects(self, user_id):
+        with sql.session_for_read() as session:
+            query = session.query(model.FederatedUser)
+            query = query.filter(model.FederatedUser.user_id == user_id)
+            fed_ref = []
+            for row in query:
+                m = model.FederatedUser(
+                    idp_id=row.idp_id,
+                    protocol_id=row.protocol_id,
+                    unique_id=row.unique_id)
+                fed_ref.append(m.to_dict())
+            return base.federated_objects_to_list(fed_ref)
+
     def _update_query_with_federated_statements(self, hints, query):
         statements = []
         for filter_ in hints.filters:
@@ -144,10 +177,8 @@ class ShadowUsers(base.ShadowUsersDriverBase):
     def create_nonlocal_user(self, user_dict):
         new_user_dict = copy.deepcopy(user_dict)
         # remove local_user attributes from new_user_dict
-        keys_to_delete = ['name', 'password']
-        for key in keys_to_delete:
-            if key in new_user_dict:
-                del new_user_dict[key]
+        new_user_dict.pop('name', None)
+        new_user_dict.pop('password', None)
         # create nonlocal_user dict
         new_nonlocal_user_dict = {
             'name': user_dict['name']
@@ -189,3 +220,32 @@ class ShadowUsers(base.ShadowUsersDriverBase):
             fed_user_refs = sql.filter_limit_query(model.FederatedUser, query,
                                                    hints)
             return [x.to_dict() for x in fed_user_refs]
+
+    def add_user_to_group_expires(self, user_id, group_id):
+        def get_federated_user():
+            with sql.session_for_read() as session:
+                query = session.query(model.FederatedUser)
+                query = query.filter_by(user_id=user_id)
+                user = query.first()
+                if not user:
+                    # Note(knikolla): This shouldn't really ever happen, since
+                    # this requires the user to already be logged in.
+                    raise exception.UserNotFound(user_id=user_id)
+                return user
+
+        with sql.session_for_write() as session:
+            user = get_federated_user()
+            query = session.query(model.ExpiringUserGroupMembership)
+            query = query.filter_by(user_id=user_id)
+            query = query.filter_by(group_id=group_id)
+            membership = query.first()
+
+            if membership:
+                membership.last_verified = datetime.datetime.utcnow()
+            else:
+                session.add(model.ExpiringUserGroupMembership(
+                    user_id=user_id,
+                    group_id=group_id,
+                    idp_id=user.idp_id,
+                    last_verified=datetime.datetime.utcnow()
+                ))
