@@ -10,11 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import flask
 import uuid
 
 from oslo_config import fixture as config_fixture
 from oslo_serialization import jsonutils
-import webob
 
 from keystone.auth.plugins import mapped
 import keystone.conf
@@ -31,25 +31,31 @@ FAKE_MAPPING_ID = uuid.uuid4().hex
 class MappingRuleEngineTests(unit.BaseTestCase):
     """A class for testing the mapping rule engine."""
 
+    def setUp(self):
+        super(MappingRuleEngineTests, self).setUp()
+        # create dummy app so we can setup a request context for our
+        # tests.
+        self.flask_app = flask.Flask(__name__)
+        self.cleanup_instance('flask_app')
+
     def assertValidMappedUserObject(self, mapped_properties,
                                     user_type='ephemeral',
                                     domain_id=None):
         """Check whether mapped properties object has 'user' within.
 
         According to today's rules, RuleProcessor does not have to issue user's
-        id or name. What's actually required is user's type and for ephemeral
-        users that would be service domain named 'Federated'.
+        id or name. What's actually required is user's type.
         """
         self.assertIn('user', mapped_properties,
                       message='Missing user object in mapped properties')
         user = mapped_properties['user']
         self.assertIn('type', user)
         self.assertEqual(user_type, user['type'])
-        self.assertIn('domain', user)
-        domain = user['domain']
-        domain_name_or_id = domain.get('id') or domain.get('name')
-        domain_ref = domain_id or 'Federated'
-        self.assertEqual(domain_ref, domain_name_or_id)
+
+        if domain_id:
+            domain = user['domain']
+            domain_name_or_id = domain.get('id') or domain.get('name')
+            self.assertEqual(domain_id, domain_name_or_id)
 
     def test_rule_engine_any_one_of_and_direct_mapping(self):
         """Should return user's name and group id EMPLOYEE_GROUP_ID.
@@ -256,6 +262,44 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         self._rule_engine_regex_match_and_many_groups(
             mapping_fixtures.MALFORMED_TESTER_ASSERTION)
 
+    def test_rule_engine_regex_blacklist(self):
+        mapping = mapping_fixtures.MAPPING_GROUPS_BLACKLIST_REGEX
+        assertion = mapping_fixtures.EMPLOYEE_PARTTIME_ASSERTION
+        rp = mapping_utils.RuleProcessor(FAKE_MAPPING_ID, mapping['rules'])
+        mapped = rp.process(assertion)
+
+        expected = {
+            'user': {'type': 'ephemeral'},
+            'projects': [],
+            'group_ids': [],
+            'group_names': [
+                {'name': 'Manager', 'domain': {
+                    'id': mapping_fixtures.FEDERATED_DOMAIN}}
+            ]
+        }
+
+        self.assertEqual(expected, mapped)
+
+    def test_rule_engine_regex_whitelist(self):
+        mapping = mapping_fixtures.MAPPING_GROUPS_WHITELIST_REGEX
+        assertion = mapping_fixtures.EMPLOYEE_PARTTIME_ASSERTION
+        rp = mapping_utils.RuleProcessor(FAKE_MAPPING_ID, mapping['rules'])
+        mapped = rp.process(assertion)
+
+        expected = {
+            'user': {'type': 'ephemeral'},
+            'projects': [],
+            'group_ids': [],
+            'group_names': [
+                {'name': 'Employee', 'domain': {
+                    'id': mapping_fixtures.FEDERATED_DOMAIN}},
+                {'name': 'PartTimeEmployee', 'domain': {
+                    'id': mapping_fixtures.FEDERATED_DOMAIN}}
+            ]
+        }
+
+        self.assertEqual(expected, mapped)
+
     def test_rule_engine_fails_after_discarding_nonstring(self):
         """Check whether RuleProcessor discards non string objects.
 
@@ -490,7 +534,7 @@ class MappingRuleEngineTests(unit.BaseTestCase):
             domain_id=mapping_fixtures.LOCAL_DOMAIN)
 
     def test_user_identifications_name(self):
-        """Test varius mapping options and how users are identified.
+        """Test various mapping options and how users are identified.
 
         This test calls mapped.setup_username() for propagating user object.
 
@@ -510,12 +554,12 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         self.assertValidMappedUserObject(mapped_properties)
         self.assertEqual('jsmith', mapped_properties['user']['name'])
         unique_id, display_name = mapped.get_user_unique_id_and_display_name(
-            {}, mapped_properties)
+            mapped_properties)
         self.assertEqual('jsmith', unique_id)
         self.assertEqual('jsmith', display_name)
 
     def test_user_identifications_name_and_federated_domain(self):
-        """Test varius mapping options and how users are identified.
+        """Test various mapping options and how users are identified.
 
         This test calls mapped.setup_username() for propagating user object.
 
@@ -533,12 +577,12 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         self.assertIsNotNone(mapped_properties)
         self.assertValidMappedUserObject(mapped_properties)
         unique_id, display_name = mapped.get_user_unique_id_and_display_name(
-            {}, mapped_properties)
+            mapped_properties)
         self.assertEqual('tbo', display_name)
         self.assertEqual('abc123%40example.com', unique_id)
 
     def test_user_identification_id(self):
-        """Test varius mapping options and how users are identified.
+        """Test various mapping options and how users are identified.
 
         This test calls mapped.setup_username() for propagating user object.
 
@@ -549,15 +593,15 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         as it was not explicitly specified in the mapping.
 
         """
-        request = webob.Request.blank('/')
         mapping = mapping_fixtures.MAPPING_USER_IDS
         rp = mapping_utils.RuleProcessor(FAKE_MAPPING_ID, mapping['rules'])
         assertion = mapping_fixtures.ADMIN_ASSERTION
         mapped_properties = rp.process(assertion)
         self.assertIsNotNone(mapped_properties)
         self.assertValidMappedUserObject(mapped_properties)
-        unique_id, display_name = mapped.get_user_unique_id_and_display_name(
-            request, mapped_properties)
+        with self.flask_app.test_request_context():
+            unique_id, display_name = (
+                mapped.get_user_unique_id_and_display_name(mapped_properties))
         self.assertEqual('bob', unique_id)
         self.assertEqual('bob', display_name)
 
@@ -566,18 +610,19 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         mapping = mapping_fixtures.MAPPING_USER_IDS
         assertion = mapping_fixtures.ADMIN_ASSERTION
         FAKE_MAPPING_ID = uuid.uuid4().hex
-        request = webob.Request.blank('/', remote_user='remote_user')
         rp = mapping_utils.RuleProcessor(FAKE_MAPPING_ID, mapping['rules'])
         mapped_properties = rp.process(assertion)
         self.assertIsNotNone(mapped_properties)
         self.assertValidMappedUserObject(mapped_properties)
-        unique_id, display_name = mapped.get_user_unique_id_and_display_name(
-            request, mapped_properties)
+        with self.flask_app.test_request_context(
+                environ_base={'REMOTE_USER': 'remote_user'}):
+            unique_id, display_name = (
+                mapped.get_user_unique_id_and_display_name(mapped_properties))
         self.assertEqual('bob', unique_id)
         self.assertEqual('remote_user', display_name)
 
     def test_user_identification_id_and_name(self):
-        """Test varius mapping options and how users are identified.
+        """Test various mapping options and how users are identified.
 
         This test calls mapped.setup_username() for propagating user object.
 
@@ -597,7 +642,6 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         not to change it.
 
         """
-        request = webob.Request.blank('/')
         testcases = [(mapping_fixtures.CUSTOMER_ASSERTION, 'bwilliams'),
                      (mapping_fixtures.EMPLOYEE_ASSERTION, 'tbo')]
         for assertion, exp_user_name in testcases:
@@ -607,8 +651,7 @@ class MappingRuleEngineTests(unit.BaseTestCase):
             self.assertIsNotNone(mapped_properties)
             self.assertValidMappedUserObject(mapped_properties)
             unique_id, display_name = (
-                mapped.get_user_unique_id_and_display_name(request,
-                                                           mapped_properties)
+                mapped.get_user_unique_id_and_display_name(mapped_properties)
             )
             self.assertEqual(exp_user_name, display_name)
             self.assertEqual('abc123%40example.com', unique_id)
@@ -670,6 +713,12 @@ class MappingRuleEngineTests(unit.BaseTestCase):
                           mapping_utils.validate_mapping_structure,
                           mapping)
 
+    def test_mapping_validation_with_bad_local_type_user_in_assertion(self):
+        mapping = mapping_fixtures.MAPPING_BAD_LOCAL_TYPE_USER_IN_ASSERTION
+        self.assertRaises(exception.ValidationError,
+                          mapping_utils.validate_mapping_structure,
+                          mapping)
+
     def test_mapping_validation_no_local(self):
         mapping = mapping_fixtures.MAPPING_MISSING_LOCAL
         self.assertRaises(exception.ValidationError,
@@ -790,6 +839,31 @@ class MappingRuleEngineTests(unit.BaseTestCase):
         ]
         self.assertEqual(expected_projects, values['projects'])
 
+    def test_rule_engine_for_groups_and_domain(self):
+        """Should return user's groups and group domain.
+
+        The GROUP_DOMAIN_ASSERTION should successfully have a match in
+        MAPPING_GROUPS_DOMAIN_OF_USER. This will test the case where a groups
+        with its domain will exist`, and return user's groups and group domain.
+
+        """
+        mapping = mapping_fixtures.MAPPING_GROUPS_DOMAIN_OF_USER
+        assertion = mapping_fixtures.GROUPS_DOMAIN_ASSERTION
+        rp = mapping_utils.RuleProcessor(FAKE_MAPPING_ID, mapping['rules'])
+        values = rp.process(assertion)
+
+        self.assertValidMappedUserObject(values)
+        user_name = assertion.get('openstack_user')
+        user_groups = ['group1', 'group2']  # since we know the input assertion
+        groups = values.get('group_names', {})
+        group_list = [g.get('name') for g in groups]
+        group_ids = values.get('group_ids')
+        name = values.get('user', {}).get('name')
+
+        self.assertEqual(user_name, name)
+        self.assertEqual(user_groups, group_list)
+        self.assertEqual([], group_ids, )
+
 
 class TestUnicodeAssertionData(unit.BaseTestCase):
     """Ensure that unicode data in the assertion headers works.
@@ -821,12 +895,14 @@ class TestUnicodeAssertionData(unit.BaseTestCase):
         # pulled from the HTTP headers. These bytes may be decodable as
         # ISO-8859-1 according to Section 3.2.4 of RFC 7230. Let's assume
         # that our web server plugins are correctly encoding the data.
-        request = webob.Request.blank(
-            '/path',
-            environ=mapping_fixtures.UNICODE_NAME_ASSERTION)
-        data = mapping_utils.get_assertion_params_from_env(request)
-        # NOTE(dstanek): keystone.auth.plugins.mapped
-        return dict(data)
+        # Create a dummy application
+        app = flask.Flask(__name__)
+        with app.test_request_context(
+                path='/path',
+                environ_overrides=mapping_fixtures.UNICODE_NAME_ASSERTION):
+            data = mapping_utils.get_assertion_params_from_env()
+            # NOTE(dstanek): keystone.auth.plugins.mapped
+            return dict(data)
 
     def test_unicode(self):
         mapping = self._pull_mapping_rules_from_the_database()
@@ -898,7 +974,6 @@ class TestMappingLocals(unit.BaseTestCase):
         expected = {
             'user': {
                 'name': 'a_user',
-                'domain': {'id': 'Federated'},
                 'type': 'ephemeral'
             },
             'projects': [],
@@ -916,7 +991,6 @@ class TestMappingLocals(unit.BaseTestCase):
         expected = {
             'user': {
                 'name': 'test_a_user',
-                'domain': {'id': 'Federated'},
                 'type': 'ephemeral'
             },
             'projects': [],
