@@ -11,17 +11,15 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
-from __future__ import absolute_import
 import atexit
 import base64
+import contextlib
 import datetime
 import functools
 import hashlib
 import json
 import ldap
 import os
-import re
 import shutil
 import socket
 import sys
@@ -29,35 +27,38 @@ import uuid
 import warnings
 
 import fixtures
+import flask
+from flask import testing as flask_testing
+import http.client
 from oslo_config import fixture as config_fixture
 from oslo_context import context as oslo_context
 from oslo_context import fixture as oslo_ctx_fixture
 from oslo_log import fixture as log_fixture
 from oslo_log import log
 from oslo_utils import timeutils
-from paste.deploy import loadwsgi
-import six
 from sqlalchemy import exc
 import testtools
 from testtools import testcase
 
+import keystone.api
 from keystone.common import context
-from keystone.common import dependency
-from keystone.common import request
+from keystone.common import json_home
+from keystone.common import provider_api
 from keystone.common import sql
 import keystone.conf
 from keystone import exception
 from keystone.identity.backends.ldap import common as ks_ldap
 from keystone import notifications
+from keystone.resource.backends import base as resource_base
+from keystone.server.flask import application as flask_app
+from keystone.server.flask import core as keystone_flask
 from keystone.tests.unit import ksfixtures
-from keystone.version import controllers
-from keystone.version import service
 
 
 keystone.conf.configure()
 keystone.conf.set_config_defaults()
 
-PID = six.text_type(os.getpid())
+PID = str(os.getpid())
 TESTSDIR = os.path.dirname(os.path.abspath(__file__))
 TESTCONF = os.path.join(TESTSDIR, 'config_files')
 ROOTDIR = os.path.normpath(os.path.join(TESTSDIR, '..', '..', '..'))
@@ -75,6 +76,7 @@ def _calc_tmpdir():
 TMPDIR = _calc_tmpdir()
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 log.register_options(CONF)
 
 IN_MEM_DB_CONN_STRING = 'sqlite://'
@@ -82,7 +84,7 @@ IN_MEM_DB_CONN_STRING = 'sqlite://'
 # Strictly matches ISO 8601 timestamps with subsecond precision like:
 # 2016-06-28T20:48:56.000000Z
 TIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
-TIME_FORMAT_REGEX = '^\d{4}-[0-1]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d{6}Z$'
+TIME_FORMAT_REGEX = r'^\d{4}-[0-1]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d{6}Z$'
 
 exception._FATAL_EXCEPTION_FORMAT_ERRORS = True
 os.makedirs(TMPDIR)
@@ -111,26 +113,6 @@ class dirs(object):
         return os.path.join(TESTCONF, *p)
 
 
-class EggLoader(loadwsgi.EggLoader):
-    _basket = {}
-
-    def find_egg_entry_point(self, object_type, name=None):
-        egg_key = '%s:%s' % (object_type, name)
-        egg_ep = self._basket.get(egg_key)
-        if not egg_ep:
-            egg_ep = super(EggLoader, self).find_egg_entry_point(
-                object_type, name=name)
-            self._basket[egg_key] = egg_ep
-        return egg_ep
-
-
-# NOTE(dstanek): class paths were remove from the keystone-paste.ini in
-# favor of using entry points. This caused tests to slow to a crawl
-# since we reload the application object for each RESTful test. This
-# monkey-patching adds caching to paste deploy's egg lookup.
-loadwsgi.EggLoader = EggLoader
-
-
 @atexit.register
 def remove_test_databases():
     db = dirs.tmp('test.db')
@@ -159,8 +141,8 @@ def skip_if_cache_disabled(*sections):
     arguments to only check global caching.
 
     If a specified configuration section does not define the `caching` option,
-    this decorator makes the same assumption as the `should_cache_fn` in
-    keystone.common.cache that caching should be enabled.
+    this decorator makes the caching enabled if `enabled` option in the `cache`
+    section of the configuration is true.
 
     """
     def wrapper(f):
@@ -257,6 +239,17 @@ def new_endpoint_ref(service_id, interface='public',
     return ref
 
 
+def new_endpoint_group_ref(filters, **kwargs):
+    ref = {
+        'id': uuid.uuid4().hex,
+        'description': uuid.uuid4().hex,
+        'filters': filters,
+        'name': uuid.uuid4().hex
+    }
+    ref.update(kwargs)
+    return ref
+
+
 def new_endpoint_ref_with_region(service_id, region, interface='public',
                                  **kwargs):
     """Define an endpoint_ref having a pre-3.2 form.
@@ -274,7 +267,9 @@ def new_domain_ref(**kwargs):
         'id': uuid.uuid4().hex,
         'name': uuid.uuid4().hex,
         'description': uuid.uuid4().hex,
-        'enabled': True
+        'enabled': True,
+        'tags': [],
+        'options': {}
     }
     ref.update(kwargs)
     return ref
@@ -288,6 +283,8 @@ def new_project_ref(domain_id=None, is_domain=False, **kwargs):
         'enabled': True,
         'domain_id': domain_id,
         'is_domain': is_domain,
+        'tags': [],
+        'options': {}
     }
     # NOTE(henry-nash): We don't include parent_id in the initial list above
     # since specifying it is optional depending on where the project sits in
@@ -323,6 +320,47 @@ def new_federated_user_ref(idp_id=None, protocol_id=None, **kwargs):
     return ref
 
 
+def new_mapping_ref(mapping_id=None, rules=None, **kwargs):
+    ref = {
+        'id': mapping_id or uuid.uuid4().hex,
+        'rules': rules or []
+    }
+    ref.update(kwargs)
+    return ref
+
+
+def new_protocol_ref(protocol_id=None, idp_id=None, mapping_id=None, **kwargs):
+    ref = {
+        'id': protocol_id or 'saml2',
+        'idp_id': idp_id or 'ORG_IDP',
+        'mapping_id': mapping_id or uuid.uuid4().hex
+    }
+    ref.update(kwargs)
+    return ref
+
+
+def new_identity_provider_ref(idp_id=None, **kwargs):
+    ref = {
+        'id': idp_id or 'ORG_IDP',
+        'enabled': True,
+        'description': '',
+    }
+    ref.update(kwargs)
+    return ref
+
+
+def new_service_provider_ref(**kwargs):
+    ref = {
+        'auth_url': 'https://' + uuid.uuid4().hex + '.com',
+        'enabled': True,
+        'description': uuid.uuid4().hex,
+        'sp_url': 'https://' + uuid.uuid4().hex + '.com',
+        'relay_state_prefix': CONF.saml.relay_state_prefix
+    }
+    ref.update(kwargs)
+    return ref
+
+
 def new_group_ref(domain_id, **kwargs):
     ref = {
         'id': uuid.uuid4().hex,
@@ -394,11 +432,41 @@ def new_totp_credential(user_id, project_id=None, blob=None):
     return credential
 
 
+def new_application_credential_ref(roles=None,
+                                   name=None,
+                                   expires=None,
+                                   secret=None):
+    ref = {
+        'id': uuid.uuid4().hex,
+        'name': uuid.uuid4().hex,
+        'description': uuid.uuid4().hex,
+    }
+    if roles:
+        ref['roles'] = roles
+    if secret:
+        ref['secret'] = secret
+
+    if isinstance(expires, str):
+        ref['expires_at'] = expires
+    elif isinstance(expires, dict):
+        ref['expires_at'] = (
+            timeutils.utcnow() + datetime.timedelta(**expires)
+        ).strftime(TIME_FORMAT)
+    elif expires is None:
+        pass
+    else:
+        raise NotImplementedError('Unexpected value for "expires"')
+
+    return ref
+
+
 def new_role_ref(**kwargs):
     ref = {
         'id': uuid.uuid4().hex,
         'name': uuid.uuid4().hex,
-        'domain_id': None
+        'description': uuid.uuid4().hex,
+        'domain_id': None,
+        'options': {},
     }
     ref.update(kwargs)
     return ref
@@ -419,6 +487,20 @@ def new_policy_ref(**kwargs):
     return ref
 
 
+def new_domain_config_ref(**kwargs):
+    ref = {
+        "identity": {
+            "driver": "ldap"
+        },
+        "ldap": {
+            "url": "ldap://myldap.com:389/",
+            "user_tree_dn": "ou=Users,dc=my_new_root,dc=org"
+        }
+    }
+    ref.update(kwargs)
+    return ref
+
+
 def new_trust_ref(trustor_user_id, trustee_user_id, project_id=None,
                   impersonation=None, expires=None, role_ids=None,
                   role_names=None, remaining_uses=None,
@@ -436,7 +518,7 @@ def new_trust_ref(trustor_user_id, trustee_user_id, project_id=None,
     if isinstance(redelegation_count, int):
         ref.update(redelegation_count=redelegation_count)
 
-    if isinstance(expires, six.string_types):
+    if isinstance(expires, str):
         ref['expires_at'] = expires
     elif isinstance(expires, dict):
         ref['expires_at'] = (
@@ -460,6 +542,30 @@ def new_trust_ref(trustor_user_id, trustee_user_id, project_id=None,
     return ref
 
 
+def new_registered_limit_ref(**kwargs):
+    ref = {
+        'service_id': uuid.uuid4().hex,
+        'resource_name': uuid.uuid4().hex,
+        'default_limit': 10,
+        'description': uuid.uuid4().hex
+    }
+
+    ref.update(kwargs)
+    return ref
+
+
+def new_limit_ref(**kwargs):
+    ref = {
+        'service_id': uuid.uuid4().hex,
+        'resource_name': uuid.uuid4().hex,
+        'resource_limit': 10,
+        'description': uuid.uuid4().hex
+    }
+
+    ref.update(kwargs)
+    return ref
+
+
 def create_user(api, domain_id, **kwargs):
     """Create a user via the API. Keep the created password.
 
@@ -474,6 +580,86 @@ def create_user(api, domain_id, **kwargs):
     return user
 
 
+def _assert_expected_status(f):
+    """Add `expected_status_code` as an argument to the test_client methods.
+
+    `expected_status_code` must be passed as a kwarg.
+    """
+    TEAPOT_HTTP_STATUS = 418
+
+    _default_expected_responses = {
+        'get': http.client.OK,
+        'head': http.client.OK,
+        'post': http.client.CREATED,
+        'put': http.client.NO_CONTENT,
+        'patch': http.client.OK,
+        'delete': http.client.NO_CONTENT,
+    }
+
+    @functools.wraps(f)
+    def inner(*args, **kwargs):
+        # Get the "expected_status_code" kwarg if supplied. If not supplied use
+        # the `_default_expected_response` mapping, or fall through to
+        # "HTTP OK" if the method is somehow unknown.
+        expected_status_code = kwargs.pop(
+            'expected_status_code',
+            _default_expected_responses.get(
+                f.__name__.lower(), http.client.OK))
+        response = f(*args, **kwargs)
+
+        # Logic to verify the response object is sane. Expand as needed
+        if response.status_code == TEAPOT_HTTP_STATUS:
+            # NOTE(morgan): We use 418 internally during tests to indicate
+            # an un-routed HTTP call was made. This allows us to avoid
+            # misinterpreting HTTP 404 from Flask and HTTP 404 from a
+            # resource that is not found (e.g. USER NOT FOUND) programmatically
+            raise AssertionError("I AM A TEAPOT(418): %s" % response.data)
+
+        if response.status_code != expected_status_code:
+            raise AssertionError(
+                'Expected HTTP Status does not match observed HTTP '
+                'Status: %(expected)s != %(observed)s (%(data)s)' % {
+                    'expected': expected_status_code,
+                    'observed': response.status_code,
+                    'data': response.data})
+
+        # return the original response object
+        return response
+    return inner
+
+
+class KeystoneFlaskTestClient(flask_testing.FlaskClient):
+    """Subclass of flask.testing.FlaskClient implementing assertions.
+
+    Implements custom "expected" HTTP Status assertion for
+    GET/HEAD/PUT/PATCH/DELETE.
+    """
+
+    @_assert_expected_status
+    def get(self, *args, **kwargs):
+        return super(KeystoneFlaskTestClient, self).get(*args, **kwargs)
+
+    @_assert_expected_status
+    def head(self, *args, **kwargs):
+        return super(KeystoneFlaskTestClient, self).head(*args, **kwargs)
+
+    @_assert_expected_status
+    def post(self, *args, **kwargs):
+        return super(KeystoneFlaskTestClient, self).post(*args, **kwargs)
+
+    @_assert_expected_status
+    def patch(self, *args, **kwargs):
+        return super(KeystoneFlaskTestClient, self).patch(*args, **kwargs)
+
+    @_assert_expected_status
+    def put(self, *args, **kwargs):
+        return super(KeystoneFlaskTestClient, self).put(*args, **kwargs)
+
+    @_assert_expected_status
+    def delete(self, *args, **kwargs):
+        return super(KeystoneFlaskTestClient, self).delete(*args, **kwargs)
+
+
 class BaseTestCase(testtools.TestCase):
     """Light weight base test class.
 
@@ -496,6 +682,9 @@ class BaseTestCase(testtools.TestCase):
         warnings.filterwarnings('error', category=DeprecationWarning,
                                 module='^keystone\\.')
         warnings.simplefilter('error', exc.SAWarning)
+        if hasattr(exc, "RemovedIn20Warning"):
+            warnings.simplefilter('ignore', exc.RemovedIn20Warning)
+
         self.addCleanup(warnings.resetwarnings)
         # Ensure we have an empty threadlocal context at the start of each
         # test.
@@ -553,6 +742,29 @@ class BaseTestCase(testtools.TestCase):
                 return True
         return False
 
+    def loadapp(self, name='public'):
+        app = flask_app.application_factory(name)
+        app.testing = True
+        app.test_client_class = KeystoneFlaskTestClient
+
+        # NOTE(morgan): any unexpected 404s, not handled by the routed apis,
+        # is a hard error and should not pass testing.
+        def page_not_found_teapot(e):
+            content = (
+                'TEST PROGRAMMING ERROR - Reached a 404 from an unrouted (`%s`'
+                ') path. Be sure the test is requesting the right resource '
+                'and that all blueprints are registered with the flask app.' %
+                flask.request.url)
+            return content, 418
+
+        app.register_error_handler(404, page_not_found_teapot)
+
+        self.test_client = app.test_client
+        self.test_request_context = app.test_request_context
+        self.cleanup_instance('test_request_context')
+        self.cleanup_instance('test_client')
+        return keystone_flask.setup_app_middleware(app)
+
 
 class TestCase(BaseTestCase):
 
@@ -560,21 +772,29 @@ class TestCase(BaseTestCase):
         return []
 
     def _policy_fixture(self):
-        return ksfixtures.Policy(dirs.etc('policy.json'), self.config_fixture)
+        return ksfixtures.Policy(self.config_fixture)
 
+    @contextlib.contextmanager
     def make_request(self, path='/', **kwargs):
+        # standup a fake app and request context with a passed in/known
+        # environment.
+
         is_admin = kwargs.pop('is_admin', False)
         environ = kwargs.setdefault('environ', {})
+        query_string = kwargs.pop('query_string', None)
+        if query_string:
+            # Make sure query string is properly added to the context
+            path = '{path}?{qs}'.format(path=path, qs=query_string)
 
         if not environ.get(context.REQUEST_CONTEXT_ENV):
             environ[context.REQUEST_CONTEXT_ENV] = context.RequestContext(
                 is_admin=is_admin,
                 authenticated=kwargs.pop('authenticated', True))
 
-        req = request.Request.blank(path=path, **kwargs)
-        req.context_dict['is_admin'] = is_admin
-
-        return req
+        # Create a dummy flask app to work with
+        app = flask.Flask(__name__)
+        with app.test_request_context(path=path, environ_overrides=environ):
+            yield
 
     def config_overrides(self):
         # NOTE(morganfainberg): enforce config_overrides can only ever be
@@ -598,16 +818,6 @@ class TestCase(BaseTestCase):
             group='catalog',
             driver='sql',
             template_file=dirs.tests('default_catalog.templates'))
-        self.config_fixture.config(
-            group='kvs',
-            backends=[
-                ('keystone.tests.unit.test_kvs.'
-                 'KVSBackendForcedKeyMangleFixture'),
-                'keystone.tests.unit.test_kvs.KVSBackendFixture'])
-        self.config_fixture.config(
-            group='signing', certfile=signing_certfile,
-            keyfile=signing_keyfile,
-            ca_certs='examples/pki/certs/cacert.pem')
         self.config_fixture.config(
             group='saml', certfile=signing_certfile, keyfile=signing_keyfile)
         self.config_fixture.config(
@@ -626,6 +836,28 @@ class TestCase(BaseTestCase):
                 'keystone.notifications=INFO',
                 'keystone.identity.backends.ldap.common=INFO',
             ])
+        # NOTE(notmorgan): Set password rounds low here to ensure speedy
+        # tests. This is explicitly set because the tests here are not testing
+        # the integrity of the password hashing, just that the correct form
+        # of hashing has been used. Note that 4 is the lowest for bcrypt
+        # allowed in the `[identity] password_hash_rounds` setting
+        self.config_fixture.config(group='identity', password_hash_rounds=4)
+
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
+
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_receipts',
+                CONF.fernet_receipts.max_active_keys
+            )
+        )
 
     def _assert_config_overrides_called(self):
         assert self.__config_overrides_called is True
@@ -647,8 +879,9 @@ class TestCase(BaseTestCase):
             keystone.conf.auth, '_register_auth_plugin_opt',
             new=mocked_register_auth_plugin_opt))
 
-        self.sql_driver_version_overrides = {}
         self.config_overrides()
+        # explicitly load auth configuration
+        keystone.conf.auth.setup_authentication()
         # NOTE(morganfainberg): ensure config_overrides has been called.
         self.addCleanup(self._assert_config_overrides_called)
 
@@ -668,20 +901,25 @@ class TestCase(BaseTestCase):
 
         # Clear the registry of providers so that providers from previous
         # tests aren't used.
-        self.addCleanup(dependency.reset)
+        self.addCleanup(provider_api.ProviderAPIs._clear_registry_instances)
+
+        # Clear the registry of JSON Home Resources
+        self.addCleanup(json_home.JsonHomeResources._reset)
 
         # Ensure Notification subscriptions and resource types are empty
         self.addCleanup(notifications.clear_subscribers)
         self.addCleanup(notifications.reset_notifier)
 
-        self.addCleanup(setattr, controllers, '_VERSIONS', [])
-
     def config(self, config_files):
         sql.initialize()
         CONF(args=[], project='keystone', default_config_files=config_files)
 
     def load_backends(self):
         """Initialize each manager and assigns them to an attribute."""
+        # TODO(morgan): Ensure our tests only ever call load_backends
+        # a single time via this method. for now just clear the registry
+        # if we are reloading.
+        provider_api.ProviderAPIs._clear_registry_instances()
         self.useFixture(ksfixtures.BackendLoader(self))
 
     def load_fixtures(self, fixtures):
@@ -700,95 +938,64 @@ class TestCase(BaseTestCase):
         if (hasattr(self, 'identity_api') and
             hasattr(self, 'assignment_api') and
                 hasattr(self, 'resource_api')):
+            try:
+                PROVIDERS.resource_api.create_domain(
+                    resource_base.NULL_DOMAIN_ID, fixtures.ROOT_DOMAIN)
+            except exception.Conflict:
+                # the root domain already exists, skip now.
+                pass
             for domain in fixtures.DOMAINS:
-                try:
-                    rv = self.resource_api.create_domain(domain['id'], domain)
-                except exception.Conflict:
-                    rv = self.resource_api.get_domain(domain['id'])
-                except exception.NotImplemented:
-                    rv = domain
+                rv = PROVIDERS.resource_api.create_domain(domain['id'], domain)
                 attrname = 'domain_%s' % domain['id']
                 setattr(self, attrname, rv)
                 fixtures_to_cleanup.append(attrname)
 
-            for tenant in fixtures.TENANTS:
-                tenant_attr_name = 'tenant_%s' % tenant['name'].lower()
-                if hasattr(self, tenant_attr_name):
-                    try:
-                        # This will clear out any roles on the project as well
-                        self.resource_api.delete_project(tenant['id'])
-                    except exception.ProjectNotFound:
-                        pass
-                rv = self.resource_api.create_project(
-                    tenant['id'], tenant)
-
-                setattr(self, tenant_attr_name, rv)
-                fixtures_to_cleanup.append(tenant_attr_name)
+            for project in fixtures.PROJECTS:
+                project_attr_name = 'project_%s' % project['name'].lower()
+                rv = PROVIDERS.resource_api.create_project(
+                    project['id'], project)
+                setattr(self, project_attr_name, rv)
+                fixtures_to_cleanup.append(project_attr_name)
 
             for role in fixtures.ROLES:
-                try:
-                    rv = self.role_api.create_role(role['id'], role)
-                except exception.Conflict:
-                    rv = self.role_api.get_role(role['id'])
-                attrname = 'role_%s' % role['id']
+                rv = PROVIDERS.role_api.create_role(role['id'], role)
+                attrname = 'role_%s' % role['name']
                 setattr(self, attrname, rv)
                 fixtures_to_cleanup.append(attrname)
 
             for user in fixtures.USERS:
                 user_copy = user.copy()
-                tenants = user_copy.pop('tenants')
-                try:
-                    existing_user = getattr(self, 'user_%s' % user['id'], None)
-                    if existing_user is not None:
-                        self.identity_api.delete_user(existing_user['id'])
-                except exception.UserNotFound:
-                    pass
+                projects = user_copy.pop('projects')
 
                 # For users, the manager layer will generate the ID
-                user_copy = self.identity_api.create_user(user_copy)
+                user_copy = PROVIDERS.identity_api.create_user(user_copy)
                 # Our tests expect that the password is still in the user
                 # record so that they can reference it, so put it back into
                 # the dict returned.
                 user_copy['password'] = user['password']
 
-                for tenant_id in tenants:
-                    try:
-                        self.assignment_api.add_user_to_project(
-                            tenant_id, user_copy['id'])
-                    except exception.Conflict:
-                        pass
+                # fixtures.ROLES[2] is the _member_ role.
+                for project_id in projects:
+                    PROVIDERS.assignment_api.add_role_to_user_and_project(
+                        user_copy['id'], project_id, fixtures.ROLES[2]['id'])
+
                 # Use the ID from the fixture as the attribute name, so
                 # that our tests can easily reference each user dict, while
                 # the ID in the dict will be the real public ID.
-                attrname = 'user_%s' % user['id']
+                attrname = 'user_%s' % user['name']
                 setattr(self, attrname, user_copy)
                 fixtures_to_cleanup.append(attrname)
 
             for role_assignment in fixtures.ROLE_ASSIGNMENTS:
                 role_id = role_assignment['role_id']
                 user = role_assignment['user']
-                tenant_id = role_assignment['tenant_id']
+                project_id = role_assignment['project_id']
                 user_id = getattr(self, 'user_%s' % user)['id']
-                try:
-                    self.assignment_api.add_role_to_user_and_project(
-                        user_id, tenant_id, role_id)
-                except exception.Conflict:
-                    pass
+                PROVIDERS.assignment_api.add_role_to_user_and_project(
+                    user_id, project_id, role_id)
 
             self.addCleanup(self.cleanup_instance(*fixtures_to_cleanup))
 
-    def _paste_config(self, config):
-        if not config.startswith('config:'):
-            test_path = os.path.join(TESTSDIR, config)
-            etc_path = os.path.join(ROOTDIR, 'etc', config)
-            for path in [test_path, etc_path]:
-                if os.path.exists('%s-paste.ini' % path):
-                    return 'config:%s-paste.ini' % path
-        return config
-
-    def loadapp(self, config, name='main'):
-        return service.loadapp(self._paste_config(config), name=name)
-
     def assertCloseEnoughForGovernmentWork(self, a, b, delta=3):
         """Assert that two datetimes are nearly equal within a small delta.
 
@@ -814,33 +1021,25 @@ class TestCase(BaseTestCase):
             "%s != %s" % (expected, value))
 
     def assertNotEmpty(self, l):
-        self.assertTrue(len(l))
+        self.assertGreater(len(l), 0)
 
-    def assertRaisesRegexp(self, expected_exception, expected_regexp,
-                           callable_obj, *args, **kwargs):
-        """Assert that the message in a raised exception matches a regexp."""
-        try:
-            callable_obj(*args, **kwargs)
-        except expected_exception as exc_value:
-            if isinstance(expected_regexp, six.string_types):
-                expected_regexp = re.compile(expected_regexp)
+    def assertUserDictEqual(self, expected, observed, message=''):
+        """Assert that a user dict is equal to another user dict.
 
-            if isinstance(exc_value.args[0], six.text_type):
-                if not expected_regexp.search(six.text_type(exc_value)):
-                    raise self.failureException(
-                        '"%s" does not match "%s"' %
-                        (expected_regexp.pattern, six.text_type(exc_value)))
-            else:
-                if not expected_regexp.search(str(exc_value)):
-                    raise self.failureException(
-                        '"%s" does not match "%s"' %
-                        (expected_regexp.pattern, str(exc_value)))
-        else:
-            if hasattr(expected_exception, '__name__'):
-                excName = expected_exception.__name__
-            else:
-                excName = str(expected_exception)
-            raise self.failureException("%s not raised" % excName)
+        User dictionaries have some variable values that should be ignored in
+        the comparison. This method is a helper that strips those elements out
+        when comparing the user dictionary. This normalized these differences
+        that should not change the comparison.
+        """
+        # NOTE(notmorgan): An empty option list is the same as no options being
+        # specified in the user_ref. This removes options if it is empty in
+        # observed if options is not specified in the expected value.
+        if ('options' in observed and not observed['options'] and
+                'options' not in expected):
+            observed = observed.copy()
+            del observed['options']
+
+        self.assertDictEqual(expected, observed, message)
 
     @property
     def ipv6_enabled(self):
@@ -872,22 +1071,4 @@ class SQLDriverOverrides(object):
         self.config_fixture.config(group='catalog', driver='sql')
         self.config_fixture.config(group='identity', driver='sql')
         self.config_fixture.config(group='policy', driver='sql')
-        self.config_fixture.config(group='token', driver='sql')
         self.config_fixture.config(group='trust', driver='sql')
-
-    def use_specific_sql_driver_version(self, driver_path,
-                                        versionless_backend, version_suffix):
-        """Add this versioned driver to the list that will be loaded.
-
-        :param driver_path: The path to the drivers, e.g. 'keystone.assignment'
-        :param versionless_backend: The name of the versionless drivers, e.g.
-                                    'backends'
-        :param version_suffix: The suffix for the version , e.g. ``V8_``
-
-        This method assumes that versioned drivers are named:
-        <version_suffix><name of versionless driver>, e.g. 'V8_backends'.
-
-        """
-        self.sql_driver_version_overrides[driver_path] = {
-            'versionless_backend': versionless_backend,
-            'versioned_backend': version_suffix + versionless_backend}
