@@ -12,11 +12,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from unittest import mock
 import uuid
 
-import mock
-from six.moves import zip
-
 from keystone.catalog.backends import base as catalog_base
 from keystone.common import provider_api
 from keystone.tests import unit
@@ -88,7 +86,8 @@ class TestTemplatedCatalog(unit.TestCase, catalog_tests.CatalogTests):
             "Templated backend doesn't have disabled endpoints")
 
     def assert_catalogs_equal(self, expected, observed):
-        sort_key = lambda d: d['id']
+        def sort_key(d):
+            return d['id']
         for e, o in zip(sorted(expected, key=sort_key),
                         sorted(observed, key=sort_key)):
             expected_endpoints = e.pop('endpoints')
@@ -249,6 +248,9 @@ class TestTemplatedCatalog(unit.TestCase, catalog_tests.CatalogTests):
     def test_invalidate_cache_when_updating_region(self):
         self.skip_test_overrides(BROKEN_WRITE_FUNCTIONALITY_MSG)
 
+    def test_update_region_extras(self):
+        self.skip_test_overrides(BROKEN_WRITE_FUNCTIONALITY_MSG)
+
     def test_create_region_with_duplicate_id(self):
         self.skip_test_overrides(BROKEN_WRITE_FUNCTIONALITY_MSG)
 
