@@ -15,20 +15,19 @@
 import datetime
 import uuid
 
+import http.client
 import oslo_context.context
 from oslo_serialization import jsonutils
-from six.moves import http_client
 from testtools import matchers
 import webtest
 
-from keystone import auth
 from keystone.common import authorization
 from keystone.common import cache
 from keystone.common import provider_api
 from keystone.common.validation import validators
 from keystone import exception
-from keystone import middleware
 from keystone.resource.backends import base as resource_base
+from keystone.server.flask.request_processing.middleware import auth_context
 from keystone.tests.common import auth as common_auth
 from keystone.tests import unit
 from keystone.tests.unit import rest
@@ -53,6 +52,8 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
                 'properties': {
                     'id': {'type': 'string', },
                     'name': {'type': 'string', },
+                    'description': {'type': 'string', },
+                    'options': {'type': 'object', }
                 },
                 'required': ['id', 'name', ],
                 'additionalProperties': False,
@@ -69,16 +70,6 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
                 'minItems': 1,
                 'maxItems': 2,
             },
-            'bind': {
-                'type': 'object',
-                'properties': {
-                    'kerberos': {
-                        'type': 'string',
-                    },
-                },
-                'required': ['kerberos'],
-                'additionalProperties': False,
-            },
             'expires_at': {
                 'type': 'string',
                 'pattern': unit.TIME_FORMAT_REGEX,
@@ -172,7 +163,7 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
             'properties': properties,
             'required': ['audit_ids', 'expires_at', 'issued_at', 'methods',
                          'user'],
-            'optional': ['bind'],
+            'optional': [],
             'additionalProperties': False
         }
 
@@ -196,11 +187,9 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         config_files.append(unit.dirs.tests_conf('backend_sql.conf'))
         return config_files
 
-    def setUp(self, app_conf='keystone', enable_sqlite_foreign_key=False):
+    def setUp(self):
         """Setup for v3 Restful Test Cases."""
-        super(RestfulTestCase, self).setUp(
-            app_conf=app_conf,
-            enable_sqlite_foreign_key=enable_sqlite_foreign_key)
+        super(RestfulTestCase, self).setUp()
 
         self.empty_context = {'environment': {}}
 
@@ -210,37 +199,27 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
 
         super(RestfulTestCase, self).load_backends()
 
-    def load_fixtures(self, fixtures, enable_sqlite_foreign_key=False):
-        self.load_sample_data(
-            enable_sqlite_foreign_key=enable_sqlite_foreign_key)
+    def load_fixtures(self, fixtures):
+        self.load_sample_data()
 
-    def _populate_default_domain(self, enable_sqlite_foreign_key=False):
+    def _populate_default_domain(self):
         try:
             PROVIDERS.resource_api.get_domain(DEFAULT_DOMAIN_ID)
         except exception.DomainNotFound:
-            # TODO(wxy): when FK is enabled in sqlite, a lot of tests will fail
-            # due to the root domain is missing. So we should open FKs for the
-            # tests one by one. If the FKs is enabled for one test,
-            # `enable_sqlite_foreign_key` should be `true` here to ensure the
-            # root domain is created. Once all tests enable FKs, the
-            # ``enable_sqlite_foreign_key`` can be removed.
-            if enable_sqlite_foreign_key:
-                root_domain = unit.new_domain_ref(
-                    id=resource_base.NULL_DOMAIN_ID,
-                    name=resource_base.NULL_DOMAIN_ID
-                )
-                self.resource_api.create_domain(resource_base.NULL_DOMAIN_ID,
-                                                root_domain)
+            root_domain = unit.new_domain_ref(
+                id=resource_base.NULL_DOMAIN_ID,
+                name=resource_base.NULL_DOMAIN_ID
+            )
+            PROVIDERS.resource_api.create_domain(resource_base.NULL_DOMAIN_ID,
+                                                 root_domain)
             domain = unit.new_domain_ref(
                 description=(u'The default domain'),
                 id=DEFAULT_DOMAIN_ID,
                 name=u'Default')
             PROVIDERS.resource_api.create_domain(DEFAULT_DOMAIN_ID, domain)
 
-    def load_sample_data(self, create_region_and_endpoints=True,
-                         enable_sqlite_foreign_key=False):
-        self._populate_default_domain(
-            enable_sqlite_foreign_key=enable_sqlite_foreign_key)
+    def load_sample_data(self, create_region_and_endpoints=True):
+        self._populate_default_domain()
         self.domain = unit.new_domain_ref()
         self.domain_id = self.domain['id']
         PROVIDERS.resource_api.create_domain(self.domain_id, self.domain)
@@ -463,7 +442,7 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         r = self.v3_create_token(auth)
         return r.headers.get('X-Subject-Token')
 
-    def v3_create_token(self, auth, expected_status=http_client.CREATED):
+    def v3_create_token(self, auth, expected_status=http.client.CREATED):
         return self.admin_request(method='POST',
                                   path='/v3/auth/tokens',
                                   body=auth,
@@ -492,29 +471,29 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
 
         return self.admin_request(path=path, token=token, **kwargs)
 
-    def get(self, path, expected_status=http_client.OK, **kwargs):
+    def get(self, path, expected_status=http.client.OK, **kwargs):
         return self.v3_request(path, method='GET',
                                expected_status=expected_status, **kwargs)
 
-    def head(self, path, expected_status=http_client.NO_CONTENT, **kwargs):
+    def head(self, path, expected_status=http.client.NO_CONTENT, **kwargs):
         r = self.v3_request(path, method='HEAD',
                             expected_status=expected_status, **kwargs)
         self.assertEqual(b'', r.body)
         return r
 
-    def post(self, path, expected_status=http_client.CREATED, **kwargs):
+    def post(self, path, expected_status=http.client.CREATED, **kwargs):
         return self.v3_request(path, method='POST',
                                expected_status=expected_status, **kwargs)
 
-    def put(self, path, expected_status=http_client.NO_CONTENT, **kwargs):
+    def put(self, path, expected_status=http.client.NO_CONTENT, **kwargs):
         return self.v3_request(path, method='PUT',
                                expected_status=expected_status, **kwargs)
 
-    def patch(self, path, expected_status=http_client.OK, **kwargs):
+    def patch(self, path, expected_status=http.client.OK, **kwargs):
         return self.v3_request(path, method='PATCH',
                                expected_status=expected_status, **kwargs)
 
-    def delete(self, path, expected_status=http_client.NO_CONTENT, **kwargs):
+    def delete(self, path, expected_status=http.client.NO_CONTENT, **kwargs):
         return self.v3_request(path, method='DELETE',
                                expected_status=expected_status, **kwargs)
 
@@ -958,8 +937,8 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
     def assertValidUser(self, entity, ref=None):
         self.assertIsNotNone(entity.get('domain_id'))
         self.assertIsNotNone(entity.get('email'))
-        self.assertIsNone(entity.get('password'))
-        self.assertNotIn('tenantId', entity)
+        self.assertNotIn('password', entity)
+        self.assertNotIn('projectId', entity)
         self.assertIn('password_expires_at', entity)
         if ref:
             self.assertEqual(ref['domain_id'], entity['domain_id'])
@@ -1096,7 +1075,7 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
 
         # Only one of user or group should be present
         if entity.get('user'):
-            self.assertIsNone(entity.get('group'))
+            self.assertNotIn('group', entity)
             self.assertIsNotNone(entity['user'].get('id'))
         else:
             self.assertIsNotNone(entity.get('group'))
@@ -1106,7 +1085,7 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         self.assertIsNotNone(entity.get('scope'))
 
         if entity['scope'].get('project'):
-            self.assertIsNone(entity['scope'].get('domain'))
+            self.assertNotIn('domain', entity['scope'])
             self.assertIsNotNone(entity['scope']['project'].get('id'))
         elif entity['scope'].get('domain'):
             self.assertIsNotNone(entity['scope'].get('domain'))
@@ -1252,18 +1231,6 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         for attribute in attributes:
             self.assertIsNotNone(entity.get(attribute))
 
-    def build_external_auth_request(self, remote_user,
-                                    remote_domain=None, auth_data=None,
-                                    kerberos=False):
-        environment = self.build_external_auth_environ(
-            remote_user, remote_domain)
-        if not auth_data:
-            auth_data = self.build_authentication_request(
-                kerberos=kerberos)['auth']
-        auth_info = auth.core.AuthInfo.create(auth_data)
-        auth_context = auth.core.AuthContext(method_names=[])
-        return self.make_request(environ=environment), auth_info, auth_context
-
     def build_external_auth_environ(self, remote_user, remote_domain=None):
         environment = {'REMOTE_USER': remote_user, 'AUTH_TYPE': 'Negotiate'}
         if remote_domain:
@@ -1289,7 +1256,7 @@ class AuthContextMiddlewareTestCase(RestfulTestCase):
             start_response('200 OK', headers)
             return [body]
 
-        app = webtest.TestApp(middleware.AuthContextMiddleware(application),
+        app = webtest.TestApp(auth_context.AuthContextMiddleware(application),
                               extra_environ=extra_environ)
         resp = app.get('/', headers={authorization.AUTH_TOKEN_HEADER: token})
         self.assertEqual(b'body', resp.body)  # just to make sure it worked
